package net.jnjmx.ch11;

import java.util.SortedSet;
import java.util.TreeSet;
import java.util.ArrayList;
import java.lang.reflect.Constructor;
import javax.management.*;
/**
 * @author - Heather Kreger
 * Java and JMX: Building Manageable Systems, Chapter 11
 * 
 * The program is provided "as is" without any warranty express or implied,
 * including the warranty of non-infringement and the implied warranties of
 * merchantibility and fitness for a particular purpose. The author will not be
 * liable for any damages suffered by you or any third party claim against 
 * you regarding the Program.
 */
/** 
 * Example illustrating how to monitor a Web service.
 * - It finds the MBeanServer for the WSTK runtime 
 * - creates MBeans for the StockQuote service and the WSTK runtime.
 * - it checks the availability of the runtime and retrieves the 
 * number of requests
 * - it checks the availability of the StockQuote service and retrieves
 * the average response time
 */

public class MonitorWebService {
 
	public static void main(String[] args) {
		String mbDomain = "SOAPManagement";
		ObjectName runtimeMBName = null, sqMBName = null, wstkMBName = null;
		MBeanServer mbs = null;
			
	 	System.out.println("Running Web Service Monitor");
		
			
		/** Find an MBeanServer for our MBean */
		// Can't do a create because our managing MBeans need to be in 
		// the same MBeanServer as the WebService and Runtime implemenations
		// MBeanServer mbs = MBeanServerFactory.createMBeanServer();
		try {
			wstkMBName = new ObjectName(mbDomain, "type", "SOAP");  // runtime MBeanName
		
			ArrayList mbServers = MBeanServerFactory.findMBeanServer(null);
			int i=0;
			boolean foundMBS = false;
			while (!mbServers.isEmpty() && !foundMBS) {
				mbs = (MBeanServer) mbServers.get(i);
				if (mbs.isRegistered(wstkMBName)) {					
					foundMBS = true;
				} else {
					i++;
				}	
			} 
			if (!foundMBS) {
				System.out.println("Could not find  WSTK MBeanServer");
			}
			
		// need to construct WSTK WS MBeanName...
		// namekeys = new Hashtable(4);
		// namekeys.put("type", "SOAP");
		// namekeys.put("name", wsName);
		// wsMBName = new ObjectName(mbDomain, namekeys);
			
		} catch (Exception e) {
			System.out.println("MonitorWebService MBean failed");
		}
		System.out.println("Found MBeanServer for Web Services Runtime");
		
		try {
			/** Create a name for the Web Service MBean for StockQuote*/	 
			runtimeMBName = new ObjectName(mbDomain, "runtime", "WSTK");
			sqMBName = new ObjectName(mbDomain, "service", "StockQuote");
			
			WebServiceManager sqmgr = new WebServiceManager("StockQuote", "V1R1");
			
			WebServiceRuntimeManager rtmgr = new WebServiceRuntimeManager("WSTK","Axis", "V3R2");
					
			// ** register the ModelMBean that is now ready to run with the MBeanServer
			ObjectInstance runtimeMB = mbs.registerMBean((Object) rtmgr, runtimeMBName);
			ObjectInstance stockQuoteMB = mbs.registerMBean((Object) sqmgr, sqMBName);
			  		
		/* server stuff */
		
//		ObjectName httpon = new ObjectName("adapters:id=Http");
//		mbs.createMBean("com.tivoli.jmx.http_pa.Listener", httpon);
//		mbs.invoke(httpon, "startListener", new Object[] {}, new String[] {}); 
		String response1, response2;
		Long rt1, rt2;
		response1 = (String) mbs.invoke(runtimeMBName, "checkAvailability", 
				new Object[] {}, 
				new String[] {});
		if (response1.equals("down")) {
			System.out.println("Web Service Runtime is not accessible");
			rt1 = new Long(0);
		}
		else {
			rt1 = (Long) mbs.getAttribute(runtimeMBName, "NumberOfRequests");
			System.out.println("Web Service Runtime is accessible through the network, " + 
			" and has received " + rt1.toString() + " requests");
		}
		
		response2 = (String) mbs.invoke(sqMBName, "checkAvailability", 
				new Object[] {}, 
				new String[] {});
		
		if ( response2.equals("down") ) {
			// if response is unavailable
			System.out.println("Web Service Stockquote is Not Available");
			rt2 = new Long(0);
		}
		else {
			// response is > 0
			rt2 = (Long) mbs.getAttribute(sqMBName, "AverageResponseTimeofResponses");
			System.out.println("Web Service Runtime is Available, processed " + rt1.toString() + " responses");
			System.out.println("Web Service Stockquote is Available, response time was " + rt2.toString() + "ms");
		}
	
		} catch (Exception e) {
			System.out.println("Web Service Monitor failed with " + e.getMessage());
		}
	}
}
