package net.jnjmx.ch11;

/**
 * @author - Heather Kreger
 * Java and JMX: Building Manageable Systems, Chapter 11
 * 
 * The program is provided "as is" without any warranty express or implied,
 * including the warranty of non-infringement and the implied warranties of
 * merchantibility and fitness for a particular purpose. The author will not be
 * liable for any damages suffered by you or any third party claim against 
 * you regarding the Program.
 */
/** 
 * Pings a hostname. 
 */
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.UnknownHostException;

public class Ping {
		/** main pings http://www.apache.org */
		public static void main(String[] args) throws Exception {
			long rc = Ping.ping("http://www.apache.org");
			System.out.println("ping=" + rc);
		}
		
		/**
		 * ping checks the network availability of a hostname
		 * >0 is good, 0 is bad, -1 is unknown host or other exception
		 * this ping uses the ping already on the system though the Runtime.
		 * This will work on a Windows 2000 system with default ping. 
		 * On a unix or using another ping tool, the parsing to extract
		 * the correct average response time my need to be customized.
		 * 
		 * @param String url to be pinged. The URL will be converted an IP address.
		 * @return long average response time
		 * */
		public static long ping(String url) { 
		/* 
		 * */
		long roundTripTime = 0;
		long timeMeasured = -1;
		long pingMS = 0;
		InetAddress uddiIPAddr;
		URL pingURL;
		String printstring;

		if (url.equals("")) {
			return -1;
		}
		// System.out.println("ping for url " + url);

		try {
			pingURL = new URL(url);
			String hostname = pingURL.getHost();	
			
			String th = (InetAddress.getLocalHost()).getHostName();
			
			uddiIPAddr = InetAddress.getByName(hostname);
			String hostipaddr = uddiIPAddr.getHostAddress();
			// System.out.println("Pinging " + hostname + " at " + hostipaddr + " from " + th);
			String[] cmdString = new String[] {"ping", hostipaddr};

			int avgindex, msindex;
			String avgms = null;
			
			Process pingproc = (Runtime.getRuntime()).exec(cmdString);
			InputStream pingos = pingproc.getInputStream();
			BufferedReader rdr = new BufferedReader(new InputStreamReader(pingos));
			printstring = rdr.readLine();
			while ((printstring != null) && (avgms == null)) {
				// System.out.println("ping returned: " + printstring);
				avgindex = printstring.indexOf("Average");				// *Average
				if (avgindex > 0) {
					msindex = printstring.indexOf("ms", avgindex);		// *Average = xxxms*
					avgms = printstring.substring(avgindex+7, msindex); // * = xxx*
				}
				printstring = rdr.readLine();
			}
			if (avgms != null) {
				avgms =  avgms.replace('=', ' ');
				avgms = avgms.trim();          // should leave just the digits
				pingMS = (new Long(avgms)).longValue();
			}
			if (pingMS <= 0) {
				return -1;
			}
			
		} catch (UnknownHostException eu) {
			System.out.println("URL " + url + " is unknown");
			return -1;
		} catch (MalformedURLException em) {
			System.out.println("URL " + url + " is malformed");
			return -1;
		} catch (Exception e) { // failed
			System.out.println("Ping failed");
			return -1;
		};
		
		return pingMS;
	}
}
	
