package net.jnjmx.ch11;

import java.net.URL;
/**
 * @author - Heather Kreger
 * Java and JMX: Building Manageable Systems, Chapter 11
 * 
 * The program is provided "as is" without any warranty express or implied,
 * including the warranty of non-infringement and the implied warranties of
 * merchantibility and fitness for a particular purpose. The author will not be
 * liable for any damages suffered by you or any third party claim against 
 * you regarding the Program.
 */
/** 
 * Standard MBean interface to manage a StockQuote service. This MBean
 * configures and sets the stock exchange for the service.
 */
public interface StockQuoteManagerMBean {
	/**
	 * getStockExchange returns the URL of the stock exchange being
	 * used by this Stock Quote service
	 * @return URL of the stock exchange
	 * */
    URL getStockExchange();
    
    /**
     * setStockExchange sets the URL of the stock exchange being
	 * used by this Stock Quote service
	 * @param URL of the stock exchange
     * */
    void setStockExchange(URL exchange);
    
    /**
     * setBackUpStockExchange sets the URL of the stock exchange to be
	 * used by this Stock Quote service if the normal exchange is not
	 * responsive or available
	 * @param URL of the stock exchange
     * */
    void setBackUpStockExchange(URL exchange);
    
    /**
     * setBackUpStockExchange gets the URL of the stock exchange to be
	 * used by this Stock Quote service if the normal exchange is not
	 * responsive or available
	 * @return URL of the stock exchange
     * */
    URL getBackUpStockExchange();
    
    /**
     * switchStockExchange changes the URL of the stock exchange to be
	 * used by this Stock Quote service right now from normal to backup
	 * or visa versa.
     * */
    void switchStockExchange();
}


