package net.jnjmx.ch11;
/**
 * @author - Heather Kreger
 * Java and JMX: Building Manageable Systems, Chapter 11
 * 
 * The program is provided "as is" without any warranty express or implied,
 * including the warranty of non-infringement and the implied warranties of
 * merchantibility and fitness for a particular purpose. The author will not be
 * liable for any damages suffered by you or any third party claim against 
 * you regarding the Program.
 */
/**
 * Standard MBean that manages a UDDI Registry as a user of the registry.
 * It provides the URL of the UDDI registry, its status, its network availability.
 *
 */
public interface UDDIRegistryClientManagerMBean {
	/**
	 * Returns the name of the UDDI registry being managed.
	 * This is for identification purposes only and is not used
	 * when interacting with the registry.
	 * @return String name of the UDDI registry
	 * */
	String getName();
	
	/**
	 * Sets the name of the UDDI registry being managed.
	 * This is for identification purposes only and is not used
	 * when interacting with the registry.
	 * @return String name of the UDDI registry
	 * */
	void setName(String name);
	
	/**
	 * Returns the inquiry URL of the UDDI registry.
	 * @return String URL of the inquiry URL for the registry
	 * */
	String getURL();
	
	/**
	 * Sets the inquiry URL of the UDDI registry.
	 * @param String URL of the inquiry URL for the registry
	 * */
	void setURL(String url);
	
	/**
	 * Returns the current availability of the UDDI Registry
	 * @return String the current status of the registry. It will
	 * be either "available" or "not available"
	 * */
	String getStatus();
	
	/**
	 * Sets the current availability of the UDDI Registry
	 * @param String desired status of the registry, or status
	 * correction. The value should be "available" or "not available"
	 * */
	void setStatus(String avail);
	
	/**
	 * checks the application availability of the UDDI Registry by 
	 * querying for businesses whose names begin with "I".
	 * If this fails, then the status should be set to "not available"
	 * If this succeeds, then the status should be set to "available"
	 * @return Long round trip time of the query, 0 if it failed,
	 * 
	 * */
	Long checkAvailability(String businessName);
	
	/**
	 * checks the network availability of the UDDI Registry by
	 * pinging the hostname. 
	 * If this fails, then the status should be set to "not available"
	 * If this succeeds, then the status should be set to "available"
	 * Note: this will not work if the URL is redirected 
	 * @return Long round trip time of the ping, 0 if it failed
	 * */
	Long checkNetworkAvailability();

}
