package net.jnjmx.ch11;

/**
 * @author - Heather Kreger
 * Java and JMX: Building Manageable Systems, Chapter 11
 * 
 * The program is provided "as is" without any warranty express or implied,
 * including the warranty of non-infringement and the implied warranties of
 * merchantibility and fitness for a particular purpose. The author will not be
 * liable for any damages suffered by you or any third party claim against 
 * you regarding the Program.
 */

/** 
 * UDDIRegistryController would be a private interface to a UDDI registry
 * that you own, host, or implement and need to control.
 * For the examples here, these methods are empty since we do not
 * own a UDDI registry and this is illustrative.
 * 
 * The methods in this class are custom operations for the UDDI Registry and will
 * create operations in the MBeans.
 * 
 * This example requires installation of UDDI4J available from
 *  http://www.uddi4j.org or http://www.ibm.com/
 */

import org.uddi4j.*;
import org.uddi4j.client.*;
import org.uddi4j.datatype.business.*;
//import org.uddi4j.request.*;
import org.uddi4j.response.*;
import org.uddi4j.util.*;
import org.w3c.dom.Element;
import java.util.Vector;
import java.util.Properties;

public class UDDIRegistryController {

	String logonID = "";
	String logonPW = "";
	String regName = "";
	String regURL = "";

	
	/**
	 * Constructor for UDDIRegistryClient.
	 */
	public UDDIRegistryController() {
	}
	
	/**
	 * Constructor for UDDIRegistryClient.
	 * @param String name of the UDDI registry
	 * @param String url inquirey URL for the UDDI registry
	 * @param String logonName logon name for a user account on the UDDI registry
	 * @param String pw password for a user account on the UDDI registry
	 */
	public UDDIRegistryController(String name, String url, String inLogonID, String inLogonPW) {
		logonID = inLogonID;
		logonPW = inLogonPW;
		regName = name;
		regURL = url;
	}
	
	/**
	 * Start the UDDI Registry
	 * These would be usefull if the UDDI Registry were under your control to 
	 * invoke a private api with the UDDI Registry's runtime to start
	 *  the UDDI registry.	
 	 * */
	public int start() { 
		return 0; 
	}
	
	/**
	 * Stop the UDDI Registry
	 * These would be usefull if the UDDI Registry were under your control to 
	 * invoke a private api with the UDDI Registry's runtime to stop
	 *  the UDDI registry.	
 	 * */
	public int stop() { 
		return 0; 
	}
	/**
	 * Backup the UDDI Registry
	 * These would be usefull if the UDDI Registry were under your control to 
	 * invoke a private api with the UDDI Registry's runtime to write the
	 * contents of the registry to a file or database.
 	 * */
	public int backup() { 
		return 0; 
	}

}
