package net.jnjmx.ch11;
import java.net.URL;
import javax.xml.namespace.QName;

//import javax.naming.Context;
//import javax.naming.InitialContext;
//import javax.xml.rpc.Service;
//import net.jnjmx.ch11.StockQuoteService;

import org.apache.axis.AxisFault;
import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import org.apache.axis.encoding.XMLType;
import org.apache.axis.utils.Options;

import javax.xml.rpc.ParameterMode;

/**
 * @author - Heather Kreger
 * Java and JMX: Building Manageable Systems, Chapter 11
 * 
 * The program is provided "as is" without any warranty express or implied,
 * including the warranty of non-infringement and the implied warranties of
 * merchantibility and fitness for a particular purpose. The author will not be
 * liable for any damages suffered by you or any third party claim against 
 * you regarding the Program.
 */
/**
 * Standard MBean manages a Web service as a client of the service and not the 
 * provider or developer of the service.
 * */
public class WebServiceClientManager 
	implements WebServiceClientManagerMBean {

	// name should be targetNamespace="http://example.com/stockquote.wsdl"
	// localpart should be portName="StockQuotePort"
	String wsName; 
	String wsURL;
	/**
	 * Constructor for WebServiceClientManager
	 * @param String name of the Web service
	 * @param String serviceURL URL of the Web service
	 * */
	public WebServiceClientManager (String name, String serviceURL) {
		wsName = name;
		wsURL = serviceURL;
	}
	
	/**
	 * @see WebServiceClientManagerMBean
	 */
	public String getName () {
		return wsName;
	}
	
	/**
	 * @see WebServiceClientManagerMBean
	 */
	public void setName(String name) {
		wsName = name;
	}
	
	/**
	 * @see WebServiceClientManagerMBean
	 * note: this example checks the availability of the
	 * StockQuoteService by invoking it to get the XXX
	 * stock price.
	 * */
	public int checkAvailability(String serviceURL) {
	  float response = 0;
	  System.out.println("WebServiceClientManager.checkAvailabilty for " + serviceURL);
	  try {
	    float xxxQuote = testQuote(new String[] {"XXX"});
      	System.out.println("WebServiceClientManager.checkAvailabilty getQuote: " + xxxQuote);
	  	if (xxxQuote > 0) return 1;
	  	else return -1;
	  /* try {
	   * JAX-RPC Variation
		Context ctx = new InitialContext();
		javax.xml.rpc.Service sqp = (Service) ctx.lookup(
				"java:comp/env/DynamicService");
		net.jnjmx.ch11.StockQuoteService sqs =
			(net.jnjmx.ch11.StockQuoteService)sqp.getPort(
						QName.valueOf(wsURL),
						StockQuoteService.class);
		response = sqs.getTradePrice("XXX", null);
	  */
	  } catch (Exception e) {
	  	System.out.println("WebServiceClientManager.checkAvailability: getStockQuote failed, service is not available");
	  	return -1;
	  }
	}
	
    /**
     * testQuote invokes the stockquote service to see if it is active
     * @param String[] of arguments, just one, the symbol to be checked: XXX
     * @return price
     * */
    public float testQuote(String args[]) throws Exception {
      Options  ins    = new Options( args );
	  String symbol;
      args = ins.getRemainingArgs();
	  System.out.println("WebServiceClientManager.testQuote");
      
      if ( args == null ) {
        System.out.println( "Usage: GetQuote <symbol>" );
        return -1;
      }

      Service  service = new Service();
      Call     call    = (Call) service.createCall();
      ins.setDefaultURL( "http://localhost:8080/axis/servlet/AxisServlet" );

      call.setTargetEndpointAddress( new URL(ins.getURL()) );
      call.setUseSOAPAction( true );
      call.setSOAPActionURI( "getQuote" );
      call.setEncodingStyle( "http://schemas.xmlsoap.org/soap/encoding/" );
      call.setOperationName( new QName("urn:xmltoday-delayed-quotes", "getQuote") );
      call.addParameter( "symbol", XMLType.XSD_STRING, ParameterMode.IN );
      call.setReturnType( XMLType.XSD_FLOAT );
      call.setUsername( ins.getUser() );
      call.setPassword( ins.getPassword() );
	  System.out.println("WebServiceClientManager.testQuote: about to call service");
      Object response = call.invoke( new Object[] { symbol = args[0] } );
	  float stkPrice = ( ((Float) response).floatValue() );
	  System.out.println("Price of stock " + symbol + " is stkPrice. Service is available");
      return stkPrice;
    }
 
	/**
	 * @see WebServiceClientManagerMBean
	 */
	public int checkNetworkAvailability (String serviceURL) {
		long respTime = Ping.ping(serviceURL);
		System.out.println("WebServiceClientManager.checkNetworkAvailability for " +
			serviceURL + ": response time is " + respTime);
		if (respTime > 0) 
			return 1;
		else 
			return -1;
	}
}

