package net.jnjmx.ch11;
import java.net.URL;
/**
 * @author - Heather Kreger
 * Java and JMX: Building Manageable Systems, Chapter 11
 * 
 * The program is provided "as is" without any warranty express or implied,
 * including the warranty of non-infringement and the implied warranties of
 * merchantibility and fitness for a particular purpose. The author will not be
 * liable for any damages suffered by you or any third party claim against 
 * you regarding the Program.
 */
/**
 * Standard MBean manages a Web service as a client of the service and not the 
 * provider or developer of the service.
 * */
public interface WebServiceClientManagerMBean {
	/**
	 * returns the name of the Web service
	 * @return String name of the Web service
	 * */
	String getName();
	
	/**
	 * sets the name of the Web service
	 * @param String name of the service
	 * */
	void setName(String name);
	
	/**
	 * checkAvailability checks that the service will respond
	 * to requests
	 * @param String url of the service to be checked
	 * @return int response time in seconds, -1 if not available
	 * */
	int checkAvailability(String serviceURL);
	
	/**
	 * checkNetworkAvailability executes a Ping.ping to the 
	 * host of the service.
	 * -1 = not available, else returns response time for ping
	 * Note, this is a simple example and will not work if redirection
	 * for the hostname portion of the service URL is done
	 * @param String serviceURL url of the service to be checked
	 * @return int response time for ping in seconds, -1 if not available
	 * 
	 */
	int checkNetworkAvailability(String serviceURL);
}
