package net.jnjmx.ch11;

import java.util.ArrayList;
import java.util.Date;
import javax.management.*;
import java.util.Hashtable;

/**
 * @author - Heather Kreger
 * Java and JMX: Building Manageable Systems, Chapter 11
 * 
 * The program is provided "as is" without any warranty express or implied,
 * including the warranty of non-infringement and the implied warranties of
 * merchantibility and fitness for a particular purpose. The author will not be
 * liable for any damages suffered by you or any third party claim against 
 * you regarding the Program.
 */
/**
 * Standard MBean manages a Web service as a provider of the service
 * */
public class WebServiceManager implements WebServiceManagerMBean {
	String wsName = "";
	String wsVersion = "v1r0";
	String mbDomain;
	Date mbInit;
	MBeanServer mbs;
	ObjectName runtimeMBName, wsMBName;
	Hashtable namekeys;
		
	/**
	 * Constructor for WebServiceManager
	 */
	public WebServiceManager() {
		mbInit = new Date();
		ArrayList mbServers = MBeanServerFactory.findMBeanServer(null);
		if (!mbServers.isEmpty())
			mbs = (MBeanServer) mbServers.get(0);
		else 
			System.out.println("WebServiceManager MBean constructor failed");
	}

	/**
	 * Constructor for WebServiceManager finds the WSTK MBeanServer and creates
	 * the WebService BMean name.
	 * @param String name of the Web service
	 * @param String serviceURL URL of the Web service
	 * */
	public WebServiceManager(String name, String version) {
		wsName = name;
		wsVersion = version;
		mbInit = new Date();
		try {
		ArrayList mbServers = MBeanServerFactory.findMBeanServer(null);
		
		if (!mbServers.isEmpty())
			mbs = (MBeanServer) mbServers.get(0);
		else 
			System.out.println("WebServiceManager MBean constructor failed");
			
		String mbDomain = "SOAPManagement";
		runtimeMBName = new ObjectName(mbDomain, "type", "SOAP");  // runtime MBeanName
		
		// need to construct WS MBeanName...
		namekeys = new Hashtable(4);
		// namekeys.put("type", "SOAP");
		namekeys.put("type", wsName);
		wsMBName = new ObjectName(mbDomain, namekeys);
		// System.out.println("WebServiceManager:1 created for " + wsMBName.toString());
			
		}
		catch (Exception e) {
			System.out.println("WebServiceManager MBean constructor failed");
		}
		
	}
	
	/**
	 * @see net.jnjmx.ch11.WebServiceManagerMBean#getName()
	 */
	public String getName() {
		return wsName;
	}
	/**
	 * @see net.jnjmx.ch11.WebServiceManagerMBean#getName()
	 */
	public void setName(String name) {
		// this will change what MBean is used, but it
		// will not change the name of the MBean.
		wsName = name;
		try {
			wsMBName = new ObjectName(mbDomain, namekeys);
		} catch (Exception e) {
			// bad name
		}
		
	}
	/**
	 * @see net.jnjmx.ch11.WebServiceManagerMBean#getVersion()
	 */
	public String getVersion() {
		// version is a local concept, not used by JMX or Web services
		return wsVersion;
	}
	/**
	 * @see net.jnjmx.ch11.WebServiceManagerMBean#setVersion()
	 */
	public void setVersion(String version) {
		wsVersion = version;
	}
		/**
	 * @see net.jnjmx.ch11.WebServiceManagerMBean#getStartDate()
	 */
	public Date getStartTime() {
		Date initDate;
		try {
			initDate = (Date) mbs.getAttribute(wsMBName, "StartDate");
		}
		catch (Exception e) {
			initDate = mbInit;
		}
		return initDate;
	}
	/**
	 * @see net.jnjmx.ch11.WebServiceManagerMBean#getNumberofRequests()
	 */
	public Integer getNumberofRequests() {
		Integer numReq;
		try {
			numReq = (Integer) mbs.getAttribute(wsMBName, "Invocations");
		}
		catch (Exception e) {
			numReq = new Integer(0);
		}
		return numReq;
	}

	/**
	 * @see net.jnjmx.ch11.WebServiceManagerMBean#getNumberofResponses()
	 */
	public Integer getNumberofResponses() {
		Integer numSucc;
		try {
			numSucc = (Integer) mbs.getAttribute(wsMBName, "Successes");
		}
		catch (Exception e) {
			numSucc = new Integer(0);
		}
		return numSucc;
	}

	/**
	 * @see net.jnjmx.ch11.WebServiceManagerMBean#getNumberofFailureResponses()
	 */
	public Integer getNumberofFailureResponses() {
		Integer numFailures; 
		try {
			numFailures = (Integer) mbs.getAttribute(wsMBName, "Failures");
		}
		catch (Exception e) {
			numFailures = new Integer(0);
		}
		return numFailures;
	}

	/**
	 * @see net.jnjmx.ch11.WebServiceManagerMBean#getAverageResponseTimeofResponses()
	 */
	public Integer getAverageResponseTimeofResponses() {
		Integer avgRT;
		try {
			avgRT = (Integer) mbs.getAttribute(wsMBName, "AverageResponseTimeMs");
		}
		catch (Exception e) {
			avgRT = new Integer(0);
		}
		return avgRT;
	}

	/**
	 * @see net.jnjmx.ch11.WebServiceManagerMBean#getAvailability()
	 */
	public String checkAvailability() {
		String status = "available";
		try {
			System.out.println("WebServiceManager.checkAvailability"
			+ " for " + wsMBName.toString());
			status = (String) mbs.getAttribute(wsMBName, "Status");
			if (status == null) status = "unknown";
		}
		catch (Exception e) {
			System.out.println("WebServiceManager.checkAvailability failed with " + e.getMessage());
			status = "unknown";
		}
		return  status;
	}

	/**
	 * @see net.jnjmx.ch11.WebServiceManagerMBean#getTotalElapsedExecutionTime()
	 */
	public Integer getTotalElapsedExecutionTime() {
		Integer procTime;
		try {
			procTime = (Integer) mbs.getAttribute(wsMBName, "TotalProcessTimeMs");
		}
		catch (Exception e) {
			procTime = new Integer(0);
		}
		return procTime;
	}

}
