package net.jnjmx.ch11;

import java.util.Date;

/**
 * @author - Heather Kreger
 * Java and JMX: Building Manageable Systems, Chapter 11
 * 
 * The program is provided "as is" without any warranty express or implied,
 * including the warranty of non-infringement and the implied warranties of
 * merchantibility and fitness for a particular purpose. The author will not be
 * liable for any damages suffered by you or any third party claim against 
 * you regarding the Program.
 */
/**
 * Standard MBean manages a Web service as a provider of the service
 * */
public interface WebServiceManagerMBean {
	/**
	 * returns the name of the Web service
	 * @return String name of the Web service
	 * */
	String getName();
	
	/**
	 * returns the version of the Web services runtime
	 * @return String version in VvRrMm.n
	 * */
	String getVersion();
	
	/**
	 * sets the version of the Web services runtime
	 * @param String version in VvRrMm.n
	 * */
	void setVersion(String version);
	
	/**
	 * returns the number of requests processed by the 
	 * Web services runtime. This is the getInvocations operation
	 * on the WSTK Server MBean.
	 * @return Integer number of requests
	 * */
	Integer getNumberofRequests();  //getInvocations
	
	/**
	 * returns the number of successfull responses returned by the 
	 * Web services runtime. This is the getSuccesses operation
	 * on the WSTK Server MBean.
	 * @return Integer number of successfull responses
	 * */
	Integer getNumberofResponses();  // getSucesses
	
	/**
	 * returns the number of failure responses processed by the 
	 * Web services runtime. This is the getFailures operation
	 * on the WSTK Server MBean.
	 * @return Integer number of failure responses
	 * */
	Integer getNumberofFailureResponses();  // getFailures
	
	/**
	 * returns the time that the Web services runtime was
	 * started. This is the getStartDate operation
	 * on the WSTK Server MBean.
	 * @return Date that WSTK runtime was started
	 * */	
	Date getStartTime();				// getStartDate
	
	/**
	 * returns the average response time for all 
	 * requests processed by the 
	 * Web services runtime. This is the getAverageResponseTimeMs
	 * operation on the WSTK Server MBean.
	 * @return Integer average responses time in milliseconds
	 * */	
	Integer getAverageResponseTimeofResponses();	// getAverageResponseTimeMs

	/**
	 * checkAvailability checks that the service will respond
	 * to requests
	 * @param String url of the service to be checked
	 * @return int response time in seconds, -1 if not available
	 * */
	String checkAvailability();				// getStatus
	
	/**
	 * returns the total CPU time Milliseconds for processing all 
	 * requests processed by the 
	 * Web services runtime. This is the getTotalProcessTimeMs
	 * operation on the WSTK Server MBean.
	 * @return Integer total processing time in milliseconds
	 * */
	Integer getTotalElapsedExecutionTime();	// getTotalProcessTimeMs

}
