package net.jnjmx.ch11;

import java.util.Date;
import java.util.ArrayList;
import java.util.Hashtable;
import javax.management.*;
/**
 * @author - Heather Kreger
 * Java and JMX: Building Manageable Systems, Chapter 11
 * 
 * The program is provided "as is" without any warranty express or implied,
 * including the warranty of non-infringement and the implied warranties of
 * merchantibility and fitness for a particular purpose. The author will not be
 * liable for any damages suffered by you or any third party claim against 
 * you regarding the Program.
 */
/**
 * Standard MBean that manages the Web Service's runtime.
 * 
 * Implementing MBean
 * 	Date getStartTime(); 			       // getStartDate
 *	Integer getNumberOfServicesDeployed(); // getServicesCount
 *	Integer getNumberofRequests();		   // getTotalCalls
 *	String getAvailability();
 *	String getServerHostname();			   // getServerHostName
 *	String getServerPort();				   // getServerPort
 *	Boolean getDebug();
 *	void setDebug(Boolean flip);
 */
public class WebServiceRuntimeManager
	implements WebServiceRuntimeManagerMBean {
		
	String rtName = "";
	String rtVersion = "V1R0";
	Date start;
	Integer numDeployed, numRequests;
	String status;
	String hostname="localhost", port="80";
	String rtType = "unknown";
	Boolean debugFlag = new Boolean("false");
	
	String mbDomain;
	Date mbInit;
	MBeanServer mbs;
	ObjectName runtimeMBName;
	
	// Hashtable namekeys;
	/**
	 * Constructor for WebServiceRuntimeManager.
	 */
	public WebServiceRuntimeManager() {
		super();
		start = new Date();
		
		mbInit = new Date();
		ArrayList mbServers = MBeanServerFactory.findMBeanServer(null);
		if (!mbServers.isEmpty())
			mbs = (MBeanServer) mbServers.get(0);
		else 
			System.out.println("WebServiceRuntimeManager MBean constructor failed");
		
	}

	/**
	 * Constructor for WebServiceRuntimeManager.
	 * Finds the BmeanSeraver
	 * @param String name of the Web Service runtime
	 * @param String type of the Web Service runtime, i.e. vendor or product
	 * @param String version of the Web service runtime
	 * */	
	public WebServiceRuntimeManager (String name, String type, String version) {
		rtName = name;
		rtType = type;
		rtVersion = version;
		start = new Date();
		try {
		ArrayList mbServers = MBeanServerFactory.findMBeanServer(null);
		
		if (!mbServers.isEmpty())
			mbs = (MBeanServer) mbServers.get(0);
		else 
			System.out.println("WebServiceRuntimeManager MBean constructor failed");
			
		String mbDomain = "SOAPManagement";
		runtimeMBName = new ObjectName(mbDomain, "type", "SOAP");  // runtime MBeanName
		
		System.out.println("WebServiceRuntimeManager constructor for WSTK's MBean " + runtimeMBName.toString());
		
		// need to construct WS MBeanName...
		// namekeys = new Hashtable(4);
		// namekeys.put("type", "SOAP");
		// namekeys.put("name", wsName);
		// wsMBName = new ObjectName(mbDomain, namekeys);
			
		}
		catch (Exception e) {
			System.out.println("WebServiceRuntimeManager MBean constructor failed");
		}
	}

	/**
	 * @see net.jnjmx.ch11.WebServiceRuntimeManagerMBean#getName()
	 */
	public String getName() {
		return rtName;
	}

	/**
	 * @see net.jnjmx.ch11.WebServiceRuntimeManagerMBean#getType()
	 */
	public String getType() {
		try {
			rtType = (String) mbs.getAttribute(runtimeMBName, "ServerType");
		} catch (Exception e) {
			// unable to get attribute, return old value
		}
		return rtType;
	}

	/**
	 * @see net.jnjmx.ch11.WebServiceRuntimeManagerMBean#getVersion()
	 */
	public String getVersion() {
		try {
			rtVersion = (String) mbs.getAttribute(runtimeMBName, "ServerVersion");
		} catch (Exception e) {
			// unable to get attribute, return old value
		}
		return rtVersion;
	}
	
	/**
	 * @see net.jnjmx.ch11.WebServiceRuntimeManagerMBean#getStartTime()
	 */
	public Date getStartTime() {
		try {
			start = (Date) mbs.getAttribute(runtimeMBName, "StartDate");
		} catch (Exception e) {
			System.out.println("WebServicesRuntimeManager.getStartTime failed " + e.getMessage());
			// unable to get attribute, return old value
		}
		return start;
	}
	
	/**
	 * @see net.jnjmx.ch11.WebServiceRuntimeManagerMBean#getNumberOfServicesDeployed()
	 */
	public Integer getNumberOfServicesDeployed() {
		try {
			numDeployed = (Integer) mbs.getAttribute(runtimeMBName, "ServicesCount");
		} catch (Exception e) {
			// unable to get attribute, return old value
		}
		return numDeployed;
	}
	
	/**
	 * @see net.jnjmx.ch11.WebServiceRuntimeManagerMBean#getNumberofRequests()
	 */
	public Integer getNumberofRequests() {
		try {
			numRequests = (Integer) mbs.getAttribute(runtimeMBName, "TotalCalls");
		} catch (Exception e) {
			System.out.println("WebServicesRuntimeManager.getNumberofRequests failed " + e.getMessage());
			// unable to get attribute, return old value
		}
		return numRequests;
	}

	/**
	 * @see net.jnjmx.ch11.WebServiceRuntimeManagerMBean#getServerHostname()
	 */
	public String getServerHostname() {
		try {
			hostname = (String) mbs.getAttribute(runtimeMBName, "ServerHostname");
		} catch (Exception e) {
			// unable to get attribute, return old value
		}
		return hostname;
	}

	/**
	 * @see net.jnjmx.ch11.WebServiceRuntimeManagerMBean#getServerPort()
	 */	
	public String getServerPort() {
		try {
			port = (String) mbs.getAttribute(runtimeMBName, "ServerPort");
		}
		catch (Exception e) {
			port = "unknown";
		}
		return port;
	}

	/**
	 * @see net.jnjmx.ch11.WebServiceRuntimeManagerMBean#getAvailability()
	 * this should exersize the Web Service Runtime in some way
	 *  In this case if it has processed requests, we'll say its available
	 *  but thats not the best test, better would be to invoke a bogus service 
	 *  through it
	 */
	public String checkAvailability() {

		// System.out.println("WebServicesRuntimeManager.checkAvailability for " + 
		//	runtimeMBName.toString());
		String status = "available";
		Integer n = getNumberofRequests();
		if ((n.intValue()) <= 0) {
			// System.out.println("WebServicesRuntimeManager.checkAvailability RT is down, num requests " + n.toString());
			status = "down";
		}
		return status;
	}
	/**
	 * @see net.jnjmx.ch11.WebServiceRuntimeManagerMBean#setDebug()
	 */
	public void setDebug(Boolean dbg) {
		try {
			Attribute newDbg = new Attribute("WstkDebug", dbg);
			mbs.setAttribute(runtimeMBName, newDbg);
		} catch (Exception e) {
			// unable to get attribute, return old value
		}
	}
	/**
	 * @see net.jnjmx.ch11.WebServiceRuntimeManagerMBean#getDebug()
	 */	
	public Boolean getDebug() {
		try {
			debugFlag = (Boolean) mbs.getAttribute(runtimeMBName, "WstkDebug");
		} catch (Exception e) {
			// unable to get attribute, return old value
		}
		return debugFlag;
	}

}
