package net.jnjmx.ch11;

import java.lang.Class;
import java.util.Date;
/**
 * @author - Heather Kreger
 * Java and JMX: Building Manageable Systems, Chapter 11
 * 
 * The program is provided "as is" without any warranty express or implied,
 * including the warranty of non-infringement and the implied warranties of
 * merchantibility and fitness for a particular purpose. The author will not be
 * liable for any damages suffered by you or any third party claim against 
 * you regarding the Program.
 */

/**
 * Standard MBean that manages the Web Service's runtime.
 * */
public interface WebServiceRuntimeManagerMBean {

	/**
	 * returns the name of the Web service
	 * @return String name of the Web service
	 * */
	String getName();
	
	/**
	 * returns the type of the Web services runtime, i.e. the
	 * vendor or product that supplied the runtime
	 * @return String type of runtime
	 * */	
	String getType();
	
	/**
	 * returns the version of the Web services runtime
	 * @return String version in VvRrMm.n
	 * */	
	String getVersion();					// getServerVersion
	
	/**
	 * returns the time that the Web services runtime was
	 * started. This is the getStartDate operation
	 * on the WSTK Server MBean.
	 * @return Date that WSTK runtime was started
	 * */	
	Date getStartTime(); 					// getStartDate

	/**
	 * returns the number of services deployed in the
	 * Web services runtime. This is the getServicesCount operation
	 * on the WSTK Server MBean.
	 * @return Integer number of services
	 * */	
	Integer getNumberOfServicesDeployed();  // getServicesCount
	/**
	 * returns the number of requests processed by the 
	 * Web services runtime. This is the getInvocations operation
	 * on the WSTK Server MBean.
	 * @return Integer number of requests
	 * */	
	Integer getNumberofRequests();			// getTotalCalls 

	/**
	 * checkAvailability checks that the service will respond
	 * to requests
	 * @param String url of the service to be checked
	 * @return int response time in seconds, -1 if not available
	 * */
	String checkAvailability();
	
	/**
	 * getServerHostname returns the host name of the
	 * Web Services runtime.
	 * @return String hostname of the runtime
	 * */
	String getServerHostname();				// getServerHostName
	
	/**
	 * getServerPort returns the port of the
	 * Web Services runtime
	 * @return String port of the runtime
	 * */
	String getServerPort();					// getServerPort
	
	/**
	 * getDebug returns the current value of the
	 * debug flag in the WSTK
	 * @return Boolean True or False Debug flag
	 * */
	Boolean getDebug();
	
	/**
	 * setDebug sets a new value for the
	 * debug flag in the WSTK
	 * @param Boolean new value True or False for Debug flag
	 * */
	void setDebug(Boolean flip);
	
	/* operations that would be nice to have but are not
	 * supported by WSTK Server MBean at this time
	int start();
	int stop();
	int deploy(String serviceName, String serviceWSDLurl, Class serviceImpl);
	int remove(String serviceName);
	int enable(String serviceName);
	int disable(String serviceName);
	*/
	
	/* WSTK Server MBean contains the following additoinal methods
   public Integer getTotalCalls();
   public Integer getTotalMessages();
   public Integer getServicesCount();
   public Date getStartDate();
   public void setServicesCount(Integer count);
   public void serviceCalled();
   public void serviceMessaged();
   public void setWstkhome(String value);
   public String getWstkhome();
   public void setServerHostname(String value);
   public String getServerHostname();
   public void setServerPort(String value);
   public String getServerPort();
   public void setUDDIRegName(String value);
   public String getUDDIRegName();
   public void setServerLocation(String value);
   public String getServerLocation();
   public void setServerType(String value);
   public String getServerType();
   public void setServerVersion(String value);
   public String getServerVersion();
   public void setWstkDebug(Boolean value);
   public Boolean getWstkDebug();
   */

}
