package net.jnjmx.ch4;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.HashMap;
import java.util.StringTokenizer;

/**
 */

public class Apache implements ApacheMBean {
	private static final String BUSY_WORKERS = "BusyWorkers";
	private static final String BYTES_PER_REQ = "BytesPerReq";
	private static final String BYTES_PER_SEC = "BytesPerSec";
	private static final String CPU_LOAD = "CPULoad";
	private static final String IDLE_WORKERS = "IdleWorkers";
	private static final String REQ_PER_SEC = "ReqPerSec";
	private static final String SCOREBOARD = "Scoreboard";
	private static final String TOTAL_ACCESSES = "Total Accesses";
	private static final String TOTAL_KBYTES = "Total kBytes";
	private static final String UPTIME = "Uptime";

	private static final int MAX_FAILURES = 3;

	public static void main(String[] args) throws Exception {
		String url = args[0];
		Apache as = new Apache(url);
		System.out.println(as.getTotalAccesses());
		Thread.sleep(500);
		System.out.println(as.getTotalAccesses());
		Thread.sleep(1000);
		System.out.println(as.getTotalAccesses());
		System.out.println(as.getBusyWorkers());
	}

	private boolean caching = true;
	private int failures = 0;
	private long lifetime = 1000;
	private String state = UNKNOWN;
	private HashMap status;
	private long tzero;
	private URL url;

	public Apache(String url)
		throws MalformedURLException, IllegalArgumentException {
		this(new URL(url));
	}

	public Apache(URL url) throws IllegalArgumentException {
		if (isStatusUrl(url) == false) {
			throw new IllegalArgumentException(url.toString());
		}
		this.status = new HashMap();
		this.url = url;
	}
	public String getServer() {
	  return this.url.toString();
	}
	
	public void setServer(String url) throws MalformedURLException, IllegalArgumentException {
	  this.url = new URL(url);
	  if (isStatusUrl(this.url) == false) {
	    throw new IllegalArgumentException(url.toString());
	  }
	  this.state = ApacheMBean.UNKNOWN;
	}
  
	public int getBusyWorkers() throws ApacheMBeanException {
		return getIntValue(BUSY_WORKERS);
	}

	public int getBytesPerSec() throws ApacheMBeanException {
		return getIntValue(BYTES_PER_SEC);
	}

	public float getBytesPerReq() throws ApacheMBeanException {
		return getFloatValue(BYTES_PER_REQ);
	}

	public long getCacheLifetime() {
		return this.lifetime;
	}

	public float getCpuLoad() throws ApacheMBeanException {
		return getFloatValue(CPU_LOAD);
	}

	public int getIdleWorkers() throws ApacheMBeanException {
		return getIntValue(IDLE_WORKERS);
	}

	public float getReqPerSec() throws ApacheMBeanException {
		return getFloatValue(REQ_PER_SEC);
	}

	public String getScoreboard() throws ApacheMBeanException {
		return getStringValue(SCOREBOARD);
	}

	public String getState() {
		System.out.println("Apache.getState() is " + this.state); 
		if ((this.state).equals("UNKNOWN")) 
			try {
				 System.out.println("Apache.getState() running getUptime to set state"); 
				 getUptime(); // connect to establish status
				 System.out.println("Apache.getState() is now " + this.state); 
			} catch (Exception e) {
				System.out.println("Apache.getState() exception " + e.getMessage());
			}
		System.out.println("Apache.getState() returning " + this.state); 
		return this.state;
	}

	public int getTotalAccesses() throws ApacheMBeanException {
		return getIntValue(TOTAL_ACCESSES);
	}

	public long getTotalKBytes() throws ApacheMBeanException {
		return getLongValue(TOTAL_KBYTES);
	}

	public long getUptime() throws ApacheMBeanException {
		return getLongValue(UPTIME);
	}

	public boolean isCaching() {
		return this.caching;
	}

	public void setCacheLifetime(long lifetime) {
		this.lifetime = lifetime;
	}

	public void setCaching(boolean caching) {
		this.caching = caching;
	}

	public String start() {
		return RUNNING;
	}

	public String stop() {
		return DOWN;
	}

	private URLConnection establishConnection()
		throws IOException, ApacheMBeanException {
		URLConnection k = this.url.openConnection();
		try {
			k.connect();
			this.failures = 0;
			this.state = RUNNING;
		} catch (IOException x) {
			if (++this.failures > MAX_FAILURES) {
				this.state = DOWN;
			} else {
				this.state = NOT_RESPONDING;
			}
			throw new ApacheMBeanException("state: " + this.state);
		}
		return k;
	}

	private void fetchStatus() throws ApacheMBeanException, IOException {
		URLConnection k = establishConnection();
		readStatus(k);
	}

	private String getValue(String value)
		throws ApacheMBeanException, NoSuchFieldException, IOException {
		String result;
		if (updateStatus()) {
			fetchStatus();
		}
		if ((result = (String) this.status.get(value)) == null) {
			throw new NoSuchFieldException(value);
		}
		return result;
	}

	private float getFloatValue(String value) throws ApacheMBeanException {
		float result;
		try {
			result = Float.parseFloat((String) getValue(value));
		} catch (IOException x) {
			throw new ApacheMBeanException(x);
		} catch (NoSuchFieldException x) {
			throw new ApacheMBeanException(x);
		}
		return result;
	}

	private int getIntValue(String value) throws ApacheMBeanException {
		int result;
		try {
			result = Integer.parseInt((String) getValue(value));
		} catch (IOException x) {
			throw new ApacheMBeanException(x);
		} catch (NoSuchFieldException x) {
			throw new ApacheMBeanException(x);
		}
		return result;
	}

	private long getLongValue(String value) throws ApacheMBeanException {
		long result;
		try {
			result = Long.parseLong((String) getValue(value));
		} catch (IOException x) {
			throw new ApacheMBeanException(x);
		} catch (NoSuchFieldException x) {
			throw new ApacheMBeanException(x);
		}
		return result;
	}

	private String getStringValue(String value) throws ApacheMBeanException {
		String result;
		try {
			result = (String) getValue(value);
		} catch (IOException x) {
			throw new ApacheMBeanException(x);
		} catch (NoSuchFieldException x) {
			throw new ApacheMBeanException(x);
		}
		return result;
	}

	private boolean isStatusUrl(URL url) {
		return url.toString().endsWith("server-status?auto");
	}

	private void readStatus(URLConnection k) throws IOException {
		BufferedReader r =
			new BufferedReader(new InputStreamReader(k.getInputStream()));
		for (String l = r.readLine(); l != null; l = r.readLine()) {
			StringTokenizer st = new StringTokenizer(l, ":");
			this.status.put(st.nextToken().trim(), st.nextToken().trim());
		}
		r.close();
		this.tzero = System.currentTimeMillis();
	}

	private boolean updateStatus() throws IOException {
		boolean result = true;
		if (this.caching) {
			long now = System.currentTimeMillis();
			result = ((now - this.tzero) > this.lifetime);
		}
		return result;
	}
}


