package net.jnjmx.ch4;

import javax.management.*;
import javax.management.modelmbean.*;

import java.lang.reflect.Constructor;
import java.util.ArrayList;

/**
 * @author Heather Kreger
 * Chapter 4:  Java and JMX Designing Manageable Systems
 * ApacheServerManager: Manages the ApacheServer as a managed
 * resource using a model MBean. 
 * 
 * ModelMBeanInfo is created using ModelMBeanInfo APIs
 * rather than an XML file.
 */

public class ApacheServerManagerWithTrace {

	public static void main(String[] args) {
		boolean tracing = false;
		String serverURL = "";
		try {
	
			 if  (args.length != 0) {
			 	System.out.println(args[0]);
			 	if (args[0] != null)
			 		 serverURL = serverURL.concat(args[0]);
			 	else 
			 		serverURL = "http://www.apache.org/server-status?auto"; 
			 	
				if ( (args[1] != null)  && (args[1]).equalsIgnoreCase("trace") ) 
			  		tracing = true;
			 }
			if (!serverURL.startsWith("http")) 
				serverURL = "http://www.apache.org/server-status?auto";
			
			if (tracing) System.out.println("Running Apache Server Manager for " + serverURL);
		
			/** Find an MBeanServer for our MBean */
			MBeanServer myMBS = MBeanServerFactory.createMBeanServer();
		    if (tracing) System.out.println("created MBeanServer");
			
			/** Instantiate an unconfigured RequiredModelMBean in the MBeanServer */
			RequiredModelMBean apacheMMBean = (RequiredModelMBean) myMBS.instantiate("javax.management.modelmbean.RequiredModelMBean");
			if (tracing) System.out.println("created ModelMBean");
			
			/** Create a name for the MBean */
			ObjectName apacheMBeanName = new ObjectName("apacheManager: id=ApacheServer");
			
			/** instantiate the ApacheServer utility to be delegated to */
			net.jnjmx.ch4.ApacheServer myApacheServer = new net.jnjmx.ch4.ApacheServer(serverURL);
			if (tracing) System.out.println("Created apache server");
			
			/** Set the configuration of the ModelMBean through the ModelMBeanInfo
			 ** This method is below */
			ApacheServerManager asmgr = new ApacheServerManager();
			ModelMBeanInfo apacheMMBeanInfo = asmgr.createApacheModelMBeanInfo(myApacheServer, apacheMBeanName);
			// ModelMBeanInfo apacheMMBeanInfo = createApacheModelMBeanInfo(myApacheServer, apacheMBeanName);
			apacheMMBean.setModelMBeanInfo(apacheMMBeanInfo);
		
			// ** Set the ApacheServer as the managed resource
			apacheMMBean.setManagedResource(myApacheServer,"ObjectReference");
		
			// ** register the ModelMBean that is now ready to run with the MBeanServer
			ObjectInstance registeredApacheMMBean = myMBS.registerMBean((Object) apacheMMBean, apacheMBeanName);
			// ** Use the ModelMBean to send the notification into JMX where it gets sent to a manager 
			// adapter that is registered for notifications from the Apache MBeans and sends them to a pager system
			
			// write ModelMBean to XML file
			XMLService xwriter = new XMLService();
			xwriter.writeXMLFile(apacheMMBean, "net/jnjmx/ch4/apachesamp1.xml");
			
			String apacheStatus = (String) apacheMMBean.invoke("getState",null, null);
			if (tracing) System.out.println("Apache Manager returned " + apacheStatus);
			
			if ( (apacheStatus.equals("DOWN") ) || (apacheStatus.equals("NOT_RESPONDING") ) ) {
			//if ( apacheStatus.equalsIgnoreCase("DOWN") ) {
				if (tracing) System.out.println("Apache Server is Down");
				apacheMMBean.sendNotification("Apache Server is Down");
				/* create monitor to notify when up again so web traffic can be routed back to it */
			}
			else 
				if (tracing) System.out.println("Apache Server is " + apacheStatus);
							
			/** create a String monitoring Mbean to monitor the apache servers status on a periodic basis */
		  	/** Ward help here */
		} catch (Exception e) {
			System.out.println("Apache Server Manager failed with " + e.getMessage());
		}
		System.exit(0);
	}
	
// Create the ModelMBeanInfo for the Apache Server. 
ModelMBeanInfo createApacheModelMBeanInfo(ApacheServer managedApache, ObjectName apacheMBeanName) {
	Class apacheClass = null;
	try {
		apacheClass = Class.forName("net.jnjmx.ch4.ApacheServer");
	} catch(Exception e) {
		System.out.println("ApacheServer Class not found");
	}

	// set MBean descriptor
	Descriptor apacheDescription = new DescriptorSupport(new String[] {
            ("name="+apacheMBeanName),    // apache server MBean name
			"descriptorType=mbean",
			("displayName=ApacheServerManager"),
			"type=net.jnjmx.ch4.ApacheServer",		// to support jakarta modeler: class of managed resource
            "log=T",					// log all notifications
			"logfile=jmxmain.log",		// in jmxmain.log
			"currencyTimeLimit=10"});	// cache all attribute values for 10 seconds

	// Define Attributes in ModelMBean AttributeInfos
	ModelMBeanAttributeInfo[] apacheAttributes = new ModelMBeanAttributeInfo[11];
	// Declare BusyWorkers attribute		
	Descriptor busyWorkersDesc = new DescriptorSupport(new String[] {
			"name=BusyWorkers", 			// attribute name
			"descriptorType=attribute",
			"displayName=Apache BusyWorkers",
			"getMethod=getBusyWorkers",	// call getValue operation for gets
			"currencyTimeLimit=3"} ); 	     	// cache BusyWorkers value
			
	apacheAttributes[0] = new ModelMBeanAttributeInfo("BusyWorkers",
			 "int",			// attribute type is String
			 "Apache Server Busy Workers",	// description of attribute
			 true,
			 false,
			 false,
		     busyWorkersDesc);				// descriptor for attribute

	// Declare BytesPerSec attribute			
	Descriptor bytesPerSecDesc = new DescriptorSupport(new String[] {
			"name=BytesPerSec", 				// attribute name
			"descriptorType=attribute",
			"displayName=Apache BytesPerSec",
			"getMethod=getBytesPerSec",  // call getValue operation for gets
			"currencyTimeLimit=10"} ); 	     	// cache BytesPerSec value
			
	apacheAttributes[1] = new ModelMBeanAttributeInfo("BytesPerSec",
			 "int",				// attribute type is String
			 "Apache Server Bytes Per Sec",		// description of attribute
			 true,
			 false,
			 false,
		     bytesPerSecDesc);				// descriptor for attribute

	// Declare BytesPerReq attribute			
	Descriptor bytesPerReqDesc = new DescriptorSupport(new String[] {
			"name=BytesPerReq", 				// attribute name
			"descriptorType=attribute",
			"displayName=Apache BytesPerReq",
			"getMethod=getBytesPerReq",	// call getValue operation for gets
			"currencyTimeLimit=10"} ); 	     	// cache BytesPerReq value
			
	apacheAttributes[2] = new ModelMBeanAttributeInfo("BytesPerReq",
			 "int",				// attribute type is Integer
			 "Apache Server Bytes Per Request",	// description of attribute
			 true,
			 false,
			 false,
		     bytesPerReqDesc);					// descriptor for attribute

	// Declare IdleWorkers attribute			
	Descriptor idleWorkersDesc = new DescriptorSupport(new String[] {
			"name=IdleWorkers", 			// attribute name
			"descriptorType=attribute",
			"displayName=Apache IdleWorkers",
			"getMethod=getIdleWorkers",	// call getValue operation for gets
			"currencyTimeLimit=10"} ); 	     	// cache IdleWorkers value
			
	apacheAttributes[3] = new ModelMBeanAttributeInfo("IdleWorkers",
			 "int",			// attribute type is Integer
			 "Apache Server Idle Workers",	// description of attribute
			 true,
			 false,
			 false,
		     idleWorkersDesc);			// descriptor for attribute

	// Declare ReqPerSec attribute			
	Descriptor reqPerSecDesc = new DescriptorSupport(new String[] {
			"name=ReqPerSec", 					// attribute name
			"descriptorType=attribute",
			"displayName=Apache ReqPerSec",
			"getMethod=getReqPerSec",	// call getValue operation for gets
			"currencyTimeLimit=5"} ); 	     	// cache ReqPerSec value
			
	apacheAttributes[4] = new ModelMBeanAttributeInfo("ReqPerSec",
			 "int",				// attribute type is Integer
			 "Apache Server Requests Per Second",	// description of attribute
			 true,
			 false,
			 false,
		     reqPerSecDesc);					// descriptor for attribute

	// Declare Scoreboard attribute			
	Descriptor ScoreboardDesc = new DescriptorSupport(new String[] {
			"name=Scoreboard", 					// attribute name
			"descriptorType=attribute",
			"displayName=Apache Scoreboard",
			"getMethod=getScoreboard",	// call getValue operation for gets
			"currencyTimeLimit=10"} ); 	     	// cache Scoreboard value
			
	apacheAttributes[5] = new ModelMBeanAttributeInfo("Scoreboard",
			 "java.lang.String",				// attribute type is String
			 "Apache Server Scoreboard",		// description of attribute
			 true,
			 false,
			 false,
		     ScoreboardDesc);					// descriptor for attribute

	// Declare TotalAccesses attribute			
	Descriptor totalAccessesDesc = new DescriptorSupport(new String[] {
			"name=TotalAccesses", 				// attribute name
			"descriptorType=attribute",
			"displayName=Apache TotalAccesses",
			"getMethod=getTotalAccesses", // call getValue operation for gets
			"currencyTimeLimit=-1"} ); 	     	// dont cache TotalAccesses value
			
	apacheAttributes[6] = new ModelMBeanAttributeInfo("TotalAccesses",
			 "int",				// attribute type is String
			 "Apache Server total accesses",	// description of attribute
			 true,
			 false,
			 false,
		     totalAccessesDesc);				// descriptor for attribute

	// Declare TotalKBytes attribute			
	Descriptor totalKBytesDesc = new DescriptorSupport(new String[] {
			"name=TotalKBytes", 				// attribute name
			"descriptorType=attribute",
			"displayName=Apache TotalKBytes",
			"getMethod=getTotalKBytes",	// call getValue operation for gets
			"currencyTimeLimit=-1"} ); 	     	// dont cache TotalKBytes value
			
	apacheAttributes[7] = new ModelMBeanAttributeInfo("TotalKBytes",
			 "int",				// attribute type is String
			 "Apache Server total KiloBytes",	// description of attribute
			 true,
			 false,
			 false,
		     totalKBytesDesc);					// descriptor for attribute

	// Declare Uptime attribute			
	Descriptor upTimeDesc = new DescriptorSupport(new String[] {
			"name=Uptime", 						// attribute name
			"descriptorType=attribute",
			"displayName=Apache Uptime",
			"getMethod=getUptime",		// call getValue operation for gets
			"currencyTimeLimit=10"} ); 	     	// cache UpTime value
			
	apacheAttributes[8] = new ModelMBeanAttributeInfo("Uptime",
			 "java.lang.String",				// attribute type is String
			 "Apache Server Up Time",			// description of attribute
			 true,
			 false,
			 false,
		     upTimeDesc);					// descriptor for attribute

	// Declare State attribute			
	Descriptor stateDesc = new DescriptorSupport(new String[] {
			"name=State", 					// attribute name
			"descriptorType=attribute",
			"displayName=Apache State",
			"getMethod=getState",			// call getValue operation for gets
			"currencyTimeLimit=10"} ); 	    // cache State value
			
	apacheAttributes[9] = new ModelMBeanAttributeInfo("State",
			 "java.lang.String",			// attribute type is String
			 "Apache Server state",			// description of attribute
			 true,
			 false,
			 false,
		     stateDesc);					// descriptor for attribute

	// Declare Server attribute			
	Descriptor serverDesc = new DescriptorSupport(new String[] {
			"name=Server", 					// attribute name
			"descriptorType=attribute",
			"displayName=Apache Server URL",
			"getMethod=getServer",			// call getServer operation for get
			"setMethod=setServer",			// call setServer operation for set
			"currencyTimeLimit=10"} ); 	   	// cache Server value
			
	apacheAttributes[10] = new ModelMBeanAttributeInfo("Server",
			 "java.lang.String",			// attribute type is String
			 "Apache Server Busy Workers",	// description of attribute
			 true,
			 true,
			 false,
		     serverDesc);				// descriptor for attribute

	// Declare constructors for the managed resource		
	Constructor[] myConstructors = apacheClass.getConstructors();
	ModelMBeanConstructorInfo[] apacheConstructors = new ModelMBeanConstructorInfo[1];
	MBeanParameterInfo[] constructorParms = new MBeanParameterInfo[] { (new MBeanParameterInfo("serverURL", 
														"java.lang.String",
														"Apache Server URL") ) };	
														
	Descriptor apacheBeanDesc = new DescriptorSupport( new String[] {
			"name=ApapcheServer",
			"descriptorType=operation", 
       			"role=constructor" } );
			
	apacheConstructors[0] = new ModelMBeanConstructorInfo("Apache",
			"ApacheServer(): Constructs a ApacheServer utility class",
			constructorParms,
//			myConstructors[0],
			apacheBeanDesc);

	// Define Operations in ModelMBeanOperationInfos
	/* Operations: getBusyWorkers, getBytesPerSec, getBytesPerReq, 
 	* getIdleWorkers, getReqPerSec, getScoreboard, getTotalAccesses, 
 	* getTotalKBytes, getUptime are satisfied by getValue, getState, 
 	* getServer, setServer(String URL), start, stop have own operations */
 	
	// Declare getValue Operation String getValue()	
	// Set paramater array	
	ModelMBeanOperationInfo[] apacheOperations = new ModelMBeanOperationInfo[6];
	MBeanParameterInfo[] getParms = new MBeanParameterInfo[0];

	MBeanParameterInfo[] getValueParms = new MBeanParameterInfo[] { (new MBeanParameterInfo("FieldName", 
														"java.lang.String",
														"Apache status field name") )};
														
	Descriptor getValueDesc = new DescriptorSupport(new String[] {
			"name=getValue",				// operation/method name
			"descriptorType=operation",
			"class=ApacheServer",			// class to run operation on
			"role=operation" } );			// this is an operation
			
	apacheOperations[0] = new ModelMBeanOperationInfo("getValue", // name
			"getValue(): get an apache status field",		// description
			getValueParms ,		// parameters for method  no parms on gets
			"java.lang.String",			// return type  of all status fields
		    MBeanOperationInfo.INFO,	// getValue does not change state
		    getValueDesc);				// descriptor for getValue

	Descriptor getStateDesc = new DescriptorSupport(new String[] {
			"name=getState",			// operation/method name
			"descriptorType=operation",
			"class=ApacheServer",			// class to run operation on
			"role=operation" } );		// this is an operation
			
	apacheOperations[1] = new ModelMBeanOperationInfo("getState", // name
			"getState(): current status of apache server",	// description
			getParms ,				// parameters for method  no parms on gets
			"java.lang.String",		// return type  status string
		    MBeanOperationInfo.INFO,	// getStatus does not change state
		    getStateDesc);			// descriptor for getStatus

	Descriptor getServerDesc = new DescriptorSupport(new String[] {
			"name=getServer",			// operation/method name
			"descriptorType=operation",
			"class=ApacheServer",			// class to run operation on
			"role=operation" } );			// this is an operation
			
	apacheOperations[2] = new ModelMBeanOperationInfo("getServer",	// name
			"getServer(): URL of apache server",			// description
			getParms ,			// parameters for method  no parms on gets
			"java.lang.Integer",		// return type  number of busy workers
		    MBeanOperationInfo.INFO,	// getServer does not change state
		    getServerDesc);			// descriptor for getServer

	MBeanParameterInfo[] setParms = new MBeanParameterInfo[] {(new MBeanParameterInfo("url", 
			"java.lang.String", 
			"Apache Server URL") )};
			
	Descriptor setServerDesc = new DescriptorSupport(new String[] {
			"name=getServer",				// operation/method name
			"descriptorType=operation",
			"class=ApacheServer",			// class to run operation on
			"role=operation" } );			// this is an operation
			
	apacheOperations[3] = new ModelMBeanOperationInfo("setServer", // name
			"getServer(): URL of apache server",		// description
			setParms ,			// parameters for method  one parms on sets
			"java.lang.String",		// return type  server url
		    MBeanOperationInfo.ACTION,	// setServer does change state
		    setServerDesc);				// descriptor for setServer

	MBeanParameterInfo[] startParms = new MBeanParameterInfo[0];
		
	Descriptor startDesc = new DescriptorSupport(new String[] {
			"name=start",				// operation/method name
			"descriptorType=operation",
			"class=ApacheServer",		// class to run operation on
			"role=operation" } );		// this is an operation
			
	apacheOperations[4] = new ModelMBeanOperationInfo("start", // name
			"start(): start apache server",	// description
			startParms ,				// parameters for method  new start value
			"java.lang.Integer",		// return type  old start value
		    MBeanOperationInfo.ACTION,	// start changes state
		    startDesc);					// descriptor for start

	MBeanParameterInfo[] stopParms = new MBeanParameterInfo[0];
	Descriptor stopDesc = new DescriptorSupport(new String[] {
			"name=stop",				// operation/method name
			"descriptorType=operation",
			"class=ApacheServer",		// class to run operation on
			"role=operation" } );		// this is an operation
			
	apacheOperations[5] = new ModelMBeanOperationInfo("stop", // name
			"stop(): start apache server",	// description
			stopParms ,			// parameters for method new stop value
			"java.lang.Integer",		// return type  old stop value
		    MBeanOperationInfo.ACTION,	// start changes state
		    stopDesc);			// descriptor for start
		    		    
	/* getters/setters for operations */
//		MBeanParameterInfo[] getParms = new MBeanParameterInfo[0];
		
	Descriptor bytespsDesc = new DescriptorSupport(new String[] {
			"name=getBytesPerSecond",				// operation/method name
			"descriptorType=operation",
			"class=ApacheServer",		// class to run operation on
			"role=operation" } );		// this is an operation
			
	apacheOperations[4] = new ModelMBeanOperationInfo("getBytesPerSecond", // name
			"number of bytes per second processed",	// description
			getParms ,				// parameters for method  new start value
			"int",		// return type  old start value
		    MBeanOperationInfo.ACTION,	// start changes state
		    bytespsDesc);					// descriptor for start
		    

	
	// Define Notifications in ModelMBeanNotificationInfo
	// declare a Serivce Outage persists notification
	ModelMBeanNotificationInfo[] apacheNotifications = new ModelMBeanNotificationInfo[1];
	Descriptor apacheDownEventDesc = new DescriptorSupport(new String[] {
			"descriptorType=notification", 
			"name=jmx.ModelMBean.General.Apache.Down", 
			"severity=1", 
			"MessageId=Apache001"});

	apacheNotifications[0] = new ModelMBeanNotificationInfo(
            new String[] {"jmx.ModelMBean.General.Apache.Down"},
			"jmx.ModelMBean.General",
			"Apache Server Down",
			apacheDownEventDesc); 


	// Create ModelMBeanInfo		
	ModelMBeanInfo apacheMMBeanInfo = new ModelMBeanInfoSupport("Apache",
		    "ModelMBean for managing an Apache Web Server",
			apacheAttributes,
			apacheConstructors,
			apacheOperations,
			apacheNotifications);
	try {		
	apacheMMBeanInfo.setMBeanDescriptor(apacheDescription);
	} catch (Exception e) {
		System.out.println("CreateMBeanInfo failed with " + e.getMessage());
	}
	return apacheMMBeanInfo;
	}


}
