package net.jnjmx.ch4;

import java.util.SortedSet;
import java.util.TreeSet;
import java.lang.reflect.Constructor;
import javax.management.*;
import javax.management.modelmbean.*;

// Managed Resource creates ModelMBeans and ModelMBeanInfo programmatically
// This shows the todd server example.
// The server is responsible for creating the ModelMBean for the Server
// The server constructor is responsible for creating the SessionPool
// and the ModelMBean for the SessionPool.

public class Server1 {
	private MBeanServer mbs;

	private SessionPool1 sessions;
	private SortedSet connectionQueue;
	private Listener listener;
	private boolean active;
	
	static boolean tracing = false;

	private long tzero;
	private int connections;

	public static void main(String[] args) throws Exception {

		try {
			 if  (args.length != 0) {
			 	System.out.println(args[0]);
			 	if ( (args[0] != null)  && (args[0]).equalsIgnoreCase("trace") ) 
			  		tracing = true;
			 }
			
		if (tracing) System.out.println("Running todd Server1");
		
		/* management stuff */
		/** Find an MBeanServer for our MBean */
		MBeanServer mbs = MBeanServerFactory.createMBeanServer();
		if (tracing) System.out.println("created MBeanServer");
		
		/** instantiate the todd server */
		Server1 server = new Server1(mbs);
		if (tracing) System.out.println("Created todd server1");  
		
		/** Create a name for the MBean */	  
		ObjectName son = new ObjectName("todd:id=Server1");
		
		/** Instantiate an unconfigured RequiredModelMBean in the MBeanServer */
		RequiredModelMBean serverMMBean = (RequiredModelMBean) mbs.instantiate("javax.management.modelmbean.RequiredModelMBean");
		if (tracing) System.out.println("created Server ModelMBean");
		
	    /** Set the configuration of the ModelMBean through the ModelMBeanInfo
		 ** This method is below */
			
		ModelMBeanInfo serverMMBeanInfo = server.createServerModelMBeanInfo(server, son);
		serverMMBean.setModelMBeanInfo(serverMMBeanInfo);
		
		// ** Set the todd Server as the managed resource
		serverMMBean.setManagedResource(server,"ObjectReference");
		
		// ** register the ModelMBean that is now ready to run with the MBeanServer
		ObjectInstance registeredServerMMBean = mbs.registerMBean((Object) serverMMBean, son);
			
  		
		/* todd server stuff */
		server.configureMonitor(mbs); /* help Ward */
		
		ObjectName httpon = new ObjectName("adapters:id=Http1");
		mbs.createMBean("com.tivoli.jmx.http_pa.Listener", httpon);
// 		mbs.invoke(httpon, "startListener", new Object[] {}, new String[] {});

		mbs.invoke(son, "start", new Object[] {}, new String[] {});
		
		System.out.println("starting connection loop");
		
		while (server.isActive()) {
			Connection k = server.waitForConnection();
			server.activateSession(k);
			if (tracing) System.out.println("*");
		}
		
		} catch (Exception e) {
			System.out.println("todd Server failed with " + e.getMessage());
		}
	 }
	
	//public Server() throws Exception {
	public Server1(MBeanServer mbs) throws Exception {
		this.mbs = mbs;

		connectionQueue = new TreeSet();
		connections = 0;
		
		// Create sessionPool
		sessions = new SessionPool1();
		
		// Create ModelMBean for SessionPool
		ObjectName spon = new ObjectName("todd:id=SessionPool1");
		RequiredModelMBean sessionMMBean = (RequiredModelMBean) mbs.instantiate("javax.management.modelmbean.RequiredModelMBean");
  		sessionMMBean.setManagedResource(sessions, "ObjectReference");
  		ModelMBeanInfo sessionMMBInfo = createSessionModelMBeanInfo(sessions, spon);
  		sessionMMBean.setModelMBeanInfo(sessionMMBInfo);
  		ObjectInstance registeredSessionMMBean = mbs.registerMBean((Object) sessionMMBean,spon);
		if (tracing) System.out.println("created Session ModelMBean");

		// the todd server is now active		
		active = true;
		tzero = System.currentTimeMillis();
	}

	public void activateSession(Connection k) {
		Session1 s;
		synchronized (sessions) {
			while ((s = sessions.reserve()) == null) {
				try {
					sessions.wait();
				} catch (InterruptedException x) {
					// see if a session is available
				}
			}
		}
		s.activate(k);
		connections++;
	}

  /** 
   * Connections attribute getter
   * @returns total number of connections handled
   */

	public Integer getConnections() {
		return new Integer(connections);
	}
	
  /**
   * Sessions attribute getter
   * @returns number of active sessions
   */

	public Integer getSessions() {
		int as = sessions.getAvailableSessions().intValue();
		int sz = sessions.getSize().intValue();
		return new Integer(sz - as);
	}
	
  /**
   * Uptime attribute getter
   * @returns number of milliseconds since the Server was started
   */

	public Long getUptime() {
		return new Long(System.currentTimeMillis() - tzero);
	}

	public boolean isActive() {
		return active;
	}
	
  /**
   * Shutdown the server killing the process and any active sessions
   */
	public void shutdown() {
		System.exit(0);
	}
	
  /**
   * 
   * Configure Monitor - Ward? What does this do?
   */
    public void configureMonitor(MBeanServer mbs) {
    }
    
  /**
   * 
   * Start a listener thread that will queue incoming connections
   */ 
	public void start() {
		listener = new Listener(connectionQueue);
		listener.start();
	}
	
  /**
   * Stop the Servers listener thread; active sessions continue to
   * handle requests
   */

	public void stop() {
		listener.stopListening();
	}

	public Connection waitForConnection() {
		Connection k;
		synchronized (connectionQueue) {
			while (connectionQueue.isEmpty()) {
				try {
					connectionQueue.wait();
				} catch (InterruptedException x) {
					// see if the queue is still empty
				}
			}
			k = (Connection) connectionQueue.first();
			connectionQueue.remove(k);
		}
		return k;
	}
	
  // Create the ModelMBeanInfo for the todd Server. 
  ModelMBeanInfo createServerModelMBeanInfo(Server1 managedServer, ObjectName serverMMBeanName) {

	Class serverClass = null;
	try {
		serverClass = Class.forName("net.jnjmx.ch4.Server1");
	} catch(Exception e) {
		System.out.println("Server Class not found");
	}

	// set MBean descriptor
	Descriptor mmbDescription = new DescriptorSupport(new String[] {
            ("name="+serverMMBeanName),    // server MBean name
			"descriptorType=mbean",
			("displayName=toddServerManager"),
            "log=T",					// log all notifications
			"logfile=jmxserver.log",	// in jmxserver.log
			"currencyTimeLimit=10"});	// cache all attribute values for 10 seconds

	// Define Attributes in ModelMBean AttributeInfos
	ModelMBeanAttributeInfo[] mmbAttributes = new ModelMBeanAttributeInfo[3];
	
	// Declare Connections attribute		
	Descriptor connectionsDesc = new DescriptorSupport(new String[] {
			"name=Connections", 		// attribute name
			"descriptorType=attribute",
			"displayName=todd connections",
			"getMethod=getConnections",	// call getConnections operation for gets
			"currencyTimeLimit=-1"} ); 	// dont cache connections value
			
	mmbAttributes[0] = new ModelMBeanAttributeInfo("Connections",
			 "java.lang.Integer",		// attribute type is String
			 "todd Server connections",	// description of attribute
			 true,
			 false,
			 false,
		     connectionsDesc);			// descriptor for attribute

	// Declare Sessions attribute
	Descriptor sessionsDesc = new DescriptorSupport(new String[] {
			"name=Sessions", 			// attribute name
			"descriptorType=attribute",
			"displayName=todd server sessions",
			"getMethod=getSessions",	// call getSessions operation for gets
			"currencyTimeLimit=5"} );	// cache sessions value for 5 seconds
			
	mmbAttributes[1] = new ModelMBeanAttributeInfo("Sessions",
			"java.lang.Integer",	// type for sessions is integer
			"todd Server sessions",
			true,
			false,
			false,
			sessionsDesc);			// descriptor for sessions
			
	// Declare upTime attribute
	Descriptor upTimeDesc = new DescriptorSupport(new String[] {
			"name=upTime", 				// attribute name
			"descriptorType=attribute",
			"displayName=todd server sessions",
			"getMethod=getUptime",		// call getUptime operation for gets
			"currencyTimeLimit=0"} );	// dont cache uptime
			
	mmbAttributes[2] = new ModelMBeanAttributeInfo("upTime",
			"java.lang.Integer",	// type for alarm is string
			"todd Server length of time up",
			true,
			false,
			false,
			upTimeDesc);			// descriptor for upTime

	// Declare constructors for MBean  HKHK or is it the managed resource????		
	ModelMBeanConstructorInfo[] mmbConstructors = new ModelMBeanConstructorInfo[1];
	Constructor[] myConstructors = serverClass.getConstructors();
	
	Descriptor mmBeanDesc = new DescriptorSupport( new String[] {
			"name=ServerClass",
			"descriptorType=operation", 
       		"role=constructor" } );
			
	mmbConstructors[0] = new ModelMBeanConstructorInfo("Server(): Constructs a todd Server",
			myConstructors[0],
			mmBeanDesc);

	// Define Operations in ModelMBeanOperationInfos
	// Operations: start, stop, shutdown 
 	// getUpTime, getConnections, getSessions
 	//
 	
	// Declare getValue Operation String getValue()	
	// Set paramater array	
	ModelMBeanOperationInfo[] mmbOperations = new ModelMBeanOperationInfo[6];

	// Declare start() Operation void start()	
	// Set paramater array	
	MBeanParameterInfo[] parms = null;
		
	Descriptor startDesc = new DescriptorSupport(new String[] {
			"name=start",			// operation/method name
			"descriptorType=operation",
			"class=server",			// class to run operation on
			"role=operation" } );	// this is an operation
			
	mmbOperations[0] = new ModelMBeanOperationInfo("start", // name
			"start server",				// description
			parms ,			// parameters for method 
			"java.lang.boolean",		// return type 
		    MBeanOperationInfo.ACTION,	// start changes state
		    startDesc);					// descriptor for enable
		    
	Descriptor stopDesc = new DescriptorSupport(new String[] {
			"name=stop",				// operation/method name
			"descriptorType=operation",
			"class=server",				// class to run operation on
			"role=operation" } );		// this is an operation
			
	mmbOperations[1] = new ModelMBeanOperationInfo("stop", // name
			"stop server",				// description
			parms ,						// parameters for method 
			"java.lang.boolean",		// return type 
		    MBeanOperationInfo.ACTION,	// stop changes state
		    stopDesc);					// descriptor for stop

	Descriptor shutdownDesc = new DescriptorSupport(new String[] {
			"name=shutdown",		// operation/method name
			"descriptorType=operation",
			"class=server",			// class to run operation on
			"role=operation" } );	// this is an operation
			
	mmbOperations[2] = new ModelMBeanOperationInfo("shutdown", // name
			"shutdown server",			// description
			parms ,						// parameters for method 
			"java.lang.boolean",		// return type 
		    MBeanOperationInfo.ACTION,	// shutdown changes state
		    shutdownDesc);				// descriptor for shutdown
			
	// Declare getConnections Operation  String getConnections()
	Descriptor getConnectionsDesc = new DescriptorSupport(new String[] {"name=getConnections",
			"descriptorType=operation",
			"class=Server",			// class to run operation on
			"role=getter"} );		// this is a getter operation

	MBeanParameterInfo[] noParms = null;			

	mmbOperations[3] = new ModelMBeanOperationInfo("getConnections", // name
			"get Connections attribute",	// description
			noParms,						// null parameter list, 
	        "java.lang.Integer",			// return type is Integer
			MBeanOperationInfo.INFO,		// does not change state
		    getConnectionsDesc);			// descriptor for getConnections
			
	// Declare getSessions Operation - void getSession()
	Descriptor getSessionsDesc = new DescriptorSupport(new String[] {
		   	"name=getSessions",		// name
			"descriptorType=operation",
			"class=Clock",			// target class for method
			"role=getter"});		// operation is a getter
						
	mmbOperations[4] = new ModelMBeanOperationInfo("getSessions",	// name
	   	    "get Sessions attribute",	// description
			noParms,					// parameters
			"java.lang.Integer",		// no return value
			MBeanOperationInfo.INFO,	// operation does not change state
			getSessionsDesc);			// getSessions descriptor
			
	// Declare getUpTime operation
	Descriptor getUpTimeDesc = new DescriptorSupport( new String[] {
			"name=getUpTime",		// name
			"descriptorType=operation",	
			"class=Server",			// target class
			"role=getter"});		// operation is a getter
			
	mmbOperations[5] = new ModelMBeanOperationInfo("getUpTime",	// name
			"get upTime attribute",	// description
			noParms,				// no parameters (null)
			"java.lang.Integer",	// return type is String
			MBeanOperationInfo.INFO,// operation does not change state
			getUpTimeDesc);			// upTime descriptor
			
	// Define no Notifications in the ModelMBeanNotificationInfo
	ModelMBeanNotificationInfo[] mmbNotifications = new ModelMBeanNotificationInfo[0];
		
	  // Create ModelMBeanInfo		
	  ModelMBeanInfo mmBeanInfo = new ModelMBeanInfoSupport("Server1", 
	  		"ModelMBeanInfo for managing a todd Server",
			mmbAttributes,
			mmbConstructors,
			mmbOperations,
			mmbNotifications);
	try {
	  mmBeanInfo.setMBeanDescriptor(mmbDescription);
	} catch (Exception e) {
		System.out.println("CreateServerMBeanInfo failed with " + e.getMessage());
	}		

	return mmBeanInfo;
	}



  ModelMBeanInfo createSessionModelMBeanInfo(SessionPool1 mmBResource, ObjectName mmbName) {
    // Create the ModelMBeanInfo for the SessionPool. 

	Class targetClass = null;
	try {
		targetClass = Class.forName("net.jnjmx.ch4.SessionPool1");
	} catch(Exception e) {
		System.out.println("Session1 Class not found");
	}

	// set MBean descriptor
	Descriptor mmbDescription = new DescriptorSupport(new String[] {
            ("name="+mmbName),    		// MBean name
			"descriptorType=mbean",
			("displayName=Session Pool Manager"),
            "log=T",					// log all notifications
			"logfile=jmxsession.log",	// in jmxsession.log
			"currencyTimeLimit=10"});	// cache all attribute values for 10 seconds

	ModelMBeanAttributeInfo[] mmbAttributes = new ModelMBeanAttributeInfo[2];
	
	// Declare Connections attribute		
	Descriptor sizeDesc = new DescriptorSupport(new String[] {
			"name=Size", 				// attribute name
			"descriptorType=attribute",
			"displayName=todd size",
			"getMethod=getSize", } ); 	
	
	mmbAttributes[0] = new ModelMBeanAttributeInfo("Size",
			 "java.lang.Integer",		// attribute type is String
			 "todd Server size",		// description of attribute
			 true,
			 false,
			 false,
		     sizeDesc);					// descriptor for attribute

	// Declare availSessions attribute
	Descriptor availSessionsDesc = new DescriptorSupport(new String[] {
			"name=AvailableSessions", 			// attribute name
			"descriptorType=attribute",
			"displayName=todd server available sessions",
			"getMethod=getAvailableSessions",	// call operation for gets
			"currencyTimeLimit=5"} );			// cache alarm value for 5 seconds
			
	mmbAttributes[1] = new ModelMBeanAttributeInfo("Sessions",
			"java.lang.Integer",		// type for sessions is integer
			"todd Server available sessions",
			true,
			false,
			false,
			availSessionsDesc);			// descriptor for sessions

	// Declare constructors for MBean  HKHK or is it the managed resource????		
	Constructor[] constructors = targetClass.getConstructors();
	ModelMBeanConstructorInfo[] mmbConstructors = new ModelMBeanConstructorInfo[1];		
	Descriptor mmBeanDesc = new DescriptorSupport( new String[] {
			"name=SessionClass",
			"descriptorType=operation", 
       		"role=constructor" } );
			
	mmbConstructors[0] = new ModelMBeanConstructorInfo("SessionPool(): Constructs a todd Server",
			constructors[0],
			mmBeanDesc);

	ModelMBeanOperationInfo[] mmbOperations = new ModelMBeanOperationInfo[3];

	// Declare grow() Operation  void grow()	
	// Set paramater array	
	MBeanParameterInfo[] noParms = null;
		
	Descriptor growDesc = new DescriptorSupport(new String[] {
			"name=grow",			// operation/method name
			"descriptorType=operation",
			"class=server",			// class to run operation on
			"role=operation" } );	// this is an operation
			
	mmbOperations[0] = new ModelMBeanOperationInfo("grow", // name
			"grow session pool",		// description
			noParms ,					// parameters for method  
			"java.lang.boolean",		// return type 
		    MBeanOperationInfo.ACTION,	// grow does change state
		    growDesc);					// descriptor for grow

		
	// Declare getSize Operation  String getSize()
	Descriptor getSizeDesc = new DescriptorSupport(new String[] {"name=getSize",
			"descriptorType=operation",
			"class=Sessions",		// class to run operation on
			"role=getter"} );		// this is a getter operation

	mmbOperations[1] = new ModelMBeanOperationInfo("getSize", // name
			"get Size attribute",	// description
			noParms,				// null parameter list, 
	        "java.lang.Integer",	// return type is Integer
			MBeanOperationInfo.INFO,// does not change state
		    getSizeDesc);			// descriptor for getSize
			
	// Declare getAvailableSessions Operation - void getSession()
	Descriptor getAvailableSessionsDesc = new DescriptorSupport(new String[] {
		   	"name=getAvailableSessions",// name
			"descriptorType=operation",
			"class=Sessions",			// target class for method
			"role=getter"});			// operation is a getter
						
	mmbOperations[2] = new ModelMBeanOperationInfo("getSessions",	// name
	   	    "get Sessions attribute",	// description
			noParms,					// parameters
			"java.lang.Integer",		// no return value
			MBeanOperationInfo.INFO,	// operation does not change state
			getAvailableSessionsDesc);	// setAvailableSessions descriptor

	ModelMBeanNotificationInfo[] mmbNotifications = new ModelMBeanNotificationInfo[0];

	// Create ModelMBeanInfo		
	ModelMBeanInfo mmBeanInfo = new ModelMBeanInfoSupport("SessionPool1",
			"ModelMBeanInfo for managing todd Sessions1",
			mmbAttributes,
			mmbConstructors,
			mmbOperations,
			mmbNotifications);
	try {
	  mmBeanInfo.setMBeanDescriptor(mmbDescription);
	} catch (Exception e) {
		System.out.println("CreateSessionPoolMBeanInfo failed with " + e.getMessage());
	}			
	return mmBeanInfo;
	}
}
