package net.jnjmx.ch4;

import java.util.SortedSet;
import java.util.TreeSet;
import java.lang.reflect.Constructor;
import javax.management.*;
import javax.management.modelmbean.*;

// Managed Server creates ModelMbeans and ModelMBeanInfo using XML files
// In this example we show how to create the same ModelMBeans using an XMLService to 
// create the ModelMBeanInfo. 
// Once again. the main loop of the Server instantiates the server and its ModelMBean
// The server's constructor instantiates the SessionPool and its ModelMBean


public class Server2 {
	private MBeanServer mbs;

	private SessionPool2 sessions;
	private SortedSet connectionQueue;
	private Listener listener;
	private boolean active;
	
	static boolean tracing = false;

	private long tzero;
	private int connections;
	
	private static XMLService XMLPrimer;

	public static void main(String[] args) throws Exception {

		try {
			 if  (args.length != 0) {
			 	System.out.println(args[0]);
			 	if ( (args[0] != null)  && (args[0]).equalsIgnoreCase("trace") ) 
			  		tracing = true;
			 }
			
		if (tracing) System.out.println("Running todd Server2");
		
		/* management stuff */
		/** Find an MBeanServer for our MBean */
		MBeanServer mbs = MBeanServerFactory.createMBeanServer();
		if (tracing) System.out.println("created MBeanServer");
		
		// instantiate the XML Service
		ObjectName sn = new ObjectName("todd:id=XMLPrimer");
		XMLPrimer = new XMLService(sn);
  		String serverMBeanInfoFile = "ServerMBean.xml";
		
		/** instantiate the todd server */
		Server2 server = new Server2(mbs);
		if (tracing) System.out.println("Created todd server");  
		
		/** Create a name for the MBean */	  
		ObjectName son = new ObjectName("todd:id=Server2");
		
	    /** Set the configuration of the ModelMBean through the 
	     * XMLPrimer service which sets ModelMBeanInfo from an
	     * XML file.
		 ** */
			
  		RequiredModelMBean serverMMBean = (RequiredModelMBean) XMLPrimer.createMBeanFromXMLFile(serverMBeanInfoFile, son);		
		/** Instantiate an unconfigured RequiredModelMBean in the MBeanServer */
		if (tracing) System.out.println("created Server ModelMBean");
		// ** Set the todd Server as the managed resource
		serverMMBean.setManagedResource(server,"ObjectReference");
		
		// ** register the ModelMBean that is now ready to run with the MBeanServer
		ObjectInstance registeredServerMMBean = mbs.registerMBean((Object) serverMMBean, son);
			  		
		/* server stuff */
		server.configureMonitor(mbs); /* help Ward */
		
		ObjectName httpon = new ObjectName("adapters:id=Http2");
		mbs.createMBean("com.tivoli.jmx.http_pa.Listener", httpon);
//		mbs.invoke(httpon, "startListener", new Object[] {}, new String[] {});

		mbs.invoke(son, "start", new Object[] {}, new String[] {});

		while (server.isActive()) {
			Connection k = server.waitForConnection();
			server.activateSession(k);
		}
		 
		} catch (Exception e) {
			System.out.println("todd Server failed with " + e.getMessage());
		}
	 }
	
	public Server2(MBeanServer mbs) throws Exception {
		this.mbs = mbs;

		connectionQueue = new TreeSet();
		connections = 0;

		sessions = new SessionPool2();
		ObjectName spon = new ObjectName("todd:id=SessionPool2");
  		  		
  		String sessionMBeanInfoFile = "SessionMBean.xml";
  		RequiredModelMBean sessionMMBean = (RequiredModelMBean) XMLPrimer.createMBeanFromXMLFile(sessionMBeanInfoFile, spon);	
  		sessionMMBean.setManagedResource(sessions, "ObjectReference");
   		ObjectInstance registeredSessionMMBean = mbs.registerMBean((Object) sessionMMBean,spon);
		if (tracing) System.out.println("created Session ModelMBean");
		
		active = true;
		tzero = System.currentTimeMillis();
	}

	public void activateSession(Connection k) {
		Session2 s;
		synchronized (sessions) {
			while ((s = sessions.reserve()) == null) {
				try {
					sessions.wait();
				} catch (InterruptedException x) {
					// see if a session is available
				}
			}
		}
		s.activate(k);
		connections++;
	}

  /** 
   * Connections attribute getter
   * @returns total number of connections handled
   */

	public Integer getConnections() {
		return new Integer(connections);
	}
	
  /**
   * Sessions attribute getter
   * @returns number of active sessions
   */

	public Integer getSessions() {
		int as = sessions.getAvailableSessions().intValue();
		int sz = sessions.getSize().intValue();
		return new Integer(sz - as);
	}
	
  /**
   * Uptime attribute getter
   * @returns number of milliseconds since the Server was started
   */

	public Long getUptime() {
		return new Long(System.currentTimeMillis() - tzero);
	}

	public boolean isActive() {
		return active;
	}
	
  /**
   * Shutdown the server killing the process and any active sessions
   */
	public void shutdown() {
		System.exit(0);
	}
	
  /**
   * 
   * Configure Monitor - Ward? What does this do?
   */
    public void configureMonitor(MBeanServer mbs) {
    }
    
  /**
   * 
   * Start a listener thread that will queue incoming connections
   */ 
	public void start() {
		listener = new Listener(connectionQueue);
		listener.start();
	}
	
  /**
   * Stop the Servers listener thread; active sessions continue to
   * handle requests
   */

	public void stop() {
		listener.stopListening();
	}

	public Connection waitForConnection() {
		Connection k;
		synchronized (connectionQueue) {
			while (connectionQueue.isEmpty()) {
				try {
					connectionQueue.wait();
				} catch (InterruptedException x) {
					// see if the queue is still empty
				}
			}
			k = (Connection) connectionQueue.first();
			connectionQueue.remove(k);
		}
		return k;
	}
	
}
