package net.jnjmx.ch4;

import java.util.HashSet;
import java.util.Set;
import java.util.ArrayList;
import javax.management.*;
import javax.management.modelmbean.*;
 
public class SessionPool4  {
	private static final int POOLSIZE = 8;

	private Set sessions;
	private int size;
	private MBeanServer mbs;

	public SessionPool4() {
		this(POOLSIZE);
	}

	public SessionPool4(int size) {
		this.size = size;
		sessions = new HashSet(this.size);
		fill();
		createSessionPoolMBean();
	}
	private void createSessionPoolMBean() {
		try {
			// change to use existing mbeanserver and existing xmlservice thru mbeanserver
		ArrayList mbsList = MBeanServerFactory.findMBeanServer(null);
		if ((!mbsList.isEmpty() && (mbsList.get(0) != null))) 
			mbs = (MBeanServer) mbsList.get(0); // use first MBeanServer
		else {
			System.out.println("SessionPool4 can't find an MBeanServer");
			System.exit(0);
		}
			
		ObjectName xmlServiceName = new ObjectName("todd:id=XMLPrimer");
		XMLService XMLPrimer = new XMLService(xmlServiceName);
		
		ObjectName spon = new ObjectName("todd:id=SessionPool");
   		String sessionXMLFile = "net/jnjmx/ch4/SessionInfo4.xml";
   		
   		// using the XMLService MBean created by Server to load ModelMBeanInfo
  		// invoke ModelMBean sessionMMBean = (ModelMBean) XMLPrimer.createMBeanFromXMLFile(sessionXMLFile, "SessionPool", this);  

  		ModelMBean sessionMMBean = (ModelMBean) mbs.invoke( xmlServiceName, 
  					"createMBeanFromXMLFile", 
  					new Object[] { sessionXMLFile, "SessionPool", (Object) this},
  					new String[] {	"java.lang.String", 
 									"java.lang.String", 
  									"java.lang.Object"} );

		// 	invoke: RequiredModelMBean sessionMMBean = (RequiredModelMBean) XMLPrimer.createMBean(mbs, xmlString, spon);

//  		RequiredModelMBean sessionMMBean = (RequiredModelMBean) mbs.invoke( xmlServiceName, "createMBean", 
// 			(new Object[] {(Object) mbs, (Object) xmlString, (Object) spon}),
//  			(new String[] {"javax.management.MBeanServer", "java.lang.String", "javax.management.ObjectName"}) );

//  		sessionMMBean.setManagedResource((Object) sessions, "ObjectReference");
   		ObjectInstance registeredSessionMMBean = mbs.registerMBean((Object) sessionMMBean,spon);
		}
		catch (Exception e) {
			System.out.println("SessionPool MBean creation failed with " + e.getMessage());
		}
	}
	
	public synchronized void grow(int increment) {
		for (int i = 0; i < increment; i++) {
			Session4 s = new Session4(this);
			sessions.add(s);
		}
		size = size + increment;
	}
	
	public Integer getSize() {
		return new Integer(size);
	}
	
	public synchronized Integer getAvailableSessions() {
		return new Integer(sessions.size());
	}

	public synchronized void release(Session4 session) {
		sessions.add(session);
		notify();
	}

	public synchronized Session4 reserve() {
		while (sessions.isEmpty()) {
			try {
				wait();
			} catch (InterruptedException x) {
				// see if the set is still empty
			}
		}
		Session4 s = (Session4) sessions.iterator().next();
		sessions.remove(s);
		return s;
	}

	private void fill() {
		for (int i = 0; i < size; i++) {
			Session4 s = new Session4(this);
			sessions.add(s);
		}
	}
}