package net.jnjmx.ch4;

import java.util.SortedSet;
import java.util.TreeSet;
import java.lang.reflect.Constructor;
import javax.management.*;
import javax.management.modelmbean.*;
// import net.jnjmx.ch4.XMLService.*;

/* 
 * This Server creates XMLModelMBeans instead of RequiredModelMBeans.
 * XMLModelMBeans extend RequiredModelMBeans and override the load
 * and store methods to write to an XML file.
 */

public class XMLServer {
	private MBeanServer mbs;

	private SessionPool sessions;
	private SortedSet connectionQueue;
	private Listener listener;
	private boolean active;
	
	static boolean tracing = false;

	private long tzero;
	private int connections;
	
	private static XMLService XMLPrimer;

	public static void main(String[] args) throws Exception {

		try {
			 if  (args.length != 0) {
			 	System.out.println(args[0]);
			 	if ( (args[0] != null)  && (args[0]).equalsIgnoreCase("trace") ) 
			  		tracing = true;
			 }
			
		if (tracing) System.out.println("Running todd Server");
		
		/* management stuff */
		/** Find an MBeanServer for our MBean */
		MBeanServer mbs = MBeanServerFactory.createMBeanServer();
		if (tracing) System.out.println("created MBeanServer");
		
		// Unlike with RequiredModelMBean, we don't need the XML
		// service because the XMLModelMBean is has a load method
		// that loads an XML file into its data
		// instantiate the XML Service

		/** instantiate the todd server */
		XMLServer server = new XMLServer(mbs);
		if (tracing) System.out.println("Created todd server");  
		
		/** Create a name for the MBean */	  
		ObjectName son = new ObjectName("todd:id=Server");
		
     	XMLModelMBean serverMMBean = (XMLModelMBean) mbs.instantiate("net.jnjmx.ch4.XMLModelMBean", 
  										new Object [] {"net/jnjmx/ch4/ServerInfo.xml", "Server"}, 
  										new String[] {"java.lang.String", "java.lang.String"});			
		//XMLModelMBean serverMMBean = new XMLModelMBean();
		//XMLService XMLPrimer = new XMLService();
		//ModelMBeanInfo mmbi = (ModelMBeanInfo) XMLPrimer.createMBeanInfoFromXMLFile("net/jnjmx/ch4/ServerInfo.xml", "Server");
		//serverMMBean.setModelMBeanInfo(mmbi);
		
		/** Instantiate an XML loaded RequiredModelMBean in the MBeanServer */
		if (tracing) System.out.println("created Server XMLModelMBean");
		
		// ** Set the todd Server as the managed resource
		serverMMBean.setManagedResource(server,"ObjectReference");
		
		// ** register the ModelMBean that is now ready to run with the MBeanServer
		ObjectInstance registeredServerMMBean = mbs.registerMBean((Object) serverMMBean, son);
		
		// test store
		Object resps = mbs.invoke(son, "store", new Object[] {}, new String[] {}); 
		System.out.println("stored");
		
		// test load
		Object respl = mbs.invoke(son, "load", new Object[] {}, new String[] {});
		System.out.println("loaded");
		
		/* server stuff */
		
//		ObjectName httpon = new ObjectName("adapters:id=Http");
//		mbs.createMBean("com.tivoli.jmx.http_pa.Listener", httpon);
//		mbs.invoke(httpon, "startListener", new Object[] {}, new String[] {}); 

		mbs.invoke(son, "getSessions", new Object[] {}, new String[] {});

//		mbs.invoke(son, "start", new Object[] {}, new String[] {});

		while (server.isActive()) {
			Connection k = server.waitForConnection();
			server.activateSession(k);
		}
		
		} catch (Exception e) {
			System.out.println("todd Server failed with " + e.getMessage());
		}
	 }
	
	//public Server() throws Exception {
	public XMLServer(MBeanServer mbs) throws Exception {
		this.mbs = mbs;

		connectionQueue = new TreeSet();
		connections = 0;

		sessions = new SessionPool(); 
		ObjectName spon = new ObjectName("todd:id=SessionPool");
  		
  		XMLModelMBean sessionMMBean = (XMLModelMBean) mbs.instantiate("net.jnjmx.ch4.XMLModelMBean", 
  										new Object [] {"net/jnjmx/ch4/SessionInfo.xml", "SessionPool"}, 
  										new String[] {"java.lang.String", "java.lang.String"});			  		
		//XMLModelMBean sessionMMBean = new XMLModelMBean("net/jnjmx/ch4/SessionInfo.xml", "id=SessionPool");
		//XMLModelMBean sessionMMBean = new XMLModelMBean("net/jnjmx/'ch4/SessionInfo.xml", "SessionPool");

		//sessionMMBean.load("net/jnjmx/ch4/SessionInfo.xml", "SessionPool");
		
		// set MBean descriptor (since it won't set using the tool)
		// ModelMBeanInfo mbi = (ModelMBeanInfo) sessionMMBean.getMBeanInfo();
		// Descriptor newMBDesc = mbi.getMBeanDescriptor();
		// newMBDesc.setField("persistLocation", "net/jnjmx/ch4");  // dir
		// newMBDesc.setField("persistName", "SessionInfo.xml");  // file
		// newMBDesc.setField("persistPolicy", "always");	 // name
		// newMBDesc.setField("name", "id=SessionPool");
		
		// mbi.setMBeanDescriptor(newMBDesc);
		// sessionMMBean.setModelMBeanInfo(mbi);
		if (tracing) System.out.println("created Session ModelMBean");
		
		sessionMMBean.store();
		sessionMMBean.setManagedResource(sessions, "ObjectReference");
   		ObjectInstance registeredSessionMMBean = mbs.registerMBean((Object) sessionMMBean,spon);
   		
		active = true;
		tzero = System.currentTimeMillis();
	}

	public void activateSession(Connection k) {
		Session s;
		synchronized (sessions) {
			while ((s = sessions.reserve()) == null) {
				try {
					sessions.wait();
				} catch (InterruptedException x) {
					// see if a session is available
				}
			}
		}
		s.activate(k);
		connections++;
	}

  /** 
   * Connections attribute getter
   * @returns total number of connections handled
   */

	public Integer getConnections() {
		return new Integer(connections);
	}
	
  /**
   * Sessions attribute getter
   * @returns number of active sessions
   */

	public Integer getSessions() {
		int as = sessions.getAvailableSessions().intValue();
		int sz = sessions.getSize().intValue();
		return new Integer(sz - as);
	}
	
  /**
   * Uptime attribute getter
   * @returns number of milliseconds since the Server was started
   */

	public Long getUptime() {
		return new Long(System.currentTimeMillis() - tzero);
	}

	public boolean isActive() {
		return active;
	}
	
  /**
   * Shutdown the server killing the process and any active sessions
   */
	public void shutdown() {
		System.exit(0);
	}
	
  /**
   * 
   * Start a listener thread that will queue incoming connections
   */ 
	public void start() {
		listener = new Listener(connectionQueue);
		listener.start();
	}
	
  /**
   * Stop the Servers listener thread; active sessions continue to
   * handle requests
   */

	public void stop() {
		listener.stopListening();
	}

	public Connection waitForConnection() {
		Connection k;
		synchronized (connectionQueue) {
			while (connectionQueue.isEmpty()) {
				try {
					connectionQueue.wait();
				} catch (InterruptedException x) {
					// see if the queue is still empty
				}
			}
			k = (Connection) connectionQueue.first();
			connectionQueue.remove(k);
		}
		return k;
	}
	
}
