package net.jnjmx.ch4;

/* Author: Heather Kreger, 
 * Book: Java and JMX: Designing Manageable Systems
 * This class contains the example descriptor fragments shown
 * in Chapter 4, section 5. This is not used anywhere else 
 * except to show that they compile.
 */
import java.lang.reflect.Constructor;
import java.util.Date;
import javax.management.*;
import javax.management.modelmbean.*;

public class createModelMBeanInfoExample {
	
	static boolean tracing = true;

	public createModelMBeanInfoExample() throws Exception {
		createModelMBeanInfo();
	}
	
	ModelMBeanInfo createModelMBeanInfo() throws Exception {
		
	// Instantiate Descriptors using the array
		Descriptor timeStateDescA = new DescriptorSupport(new String[] {
                    "name=Time",
					"descriptorType=attribute",
					"displayName=currentTime",
                    "getMethod=getTime",
					"setMethod=setTime",
					"currencyTimeLimit=20" });

	// 	Instantiate Descriptors using the field setters
		Descriptor timeStateDescF = new DescriptorSupport();
		timeStateDescF.setField("name","Time");
		timeStateDescF.setField("descriptorType","attribute");
		timeStateDescF.setField("displayName","currentTime");
		timeStateDescF.setField("getMethod","getTime");
		timeStateDescF.setField("setMethod","setTime");
		timeStateDescF.setField("currencyTimeLimit","20");

	// set the cached value for time
		Date currDate = new Date();
		timeStateDescF.setField("value", currDate);

	// Instantiate using Field Arrays
	
		Descriptor timeStateDescFA = new DescriptorSupport();
		String[] fieldnames = new String[] {"name", 
			"descriptorType", 
			"displayName",
			"getMethod", 
			"setMethod",
			"currencyTimeLimit"} ;
		Object[] fieldvalues = new Object[] {
			new String("Time"),
			new String("attribute"),
			new String("currentTime"),
			new String("getTime"),
			new String("setTime"),
			new String("20") } ;
		timeStateDescFA.setFields(fieldnames,fieldvalues);

	// 	set MBean descriptor example
		Descriptor mmbDescription = new DescriptorSupport(new String[] {
                ("name=serverManager"),			// server MBean name
				"descriptorType=mbean",			// mbean descriptor
				("displayName=serverMBean"),	// screen name of MBean
                "log=T",						// log all notifications
				"logfile=jmxmain.log",			// log in jmxmain.log
				"currencyTimeLimit=10",	// cache all attribute values for 10 seconds
				"persistPolicy=noMoreOftenThan",	// persist on change
				"persistPeriod=10",				// no more often tan 10 seconds
				"persistLocation=jmxRepository",// save in jmxRepository Directory
				"persistName=serverBean"});		// save in file called serverMBean

	// Set Attribute Descriptors Examples
	
	ModelMBeanAttributeInfo[] dAttributes = new ModelMBeanAttributeInfo[3];
	
	// State

		Descriptor stateDesc = new DescriptorSupport();
			stateDesc.setField("name","State");
			stateDesc.setField("descriptorType","attribute");
			stateDesc.setField("displayName","Apache Server State");
			stateDesc.setField("getMethod","getState");
			stateDesc.setField("setMethod","setState");
			stateDesc.setField("currencyTimeLimit","20");
			
		dAttributes[0] = new ModelMBeanAttributeInfo("State",
				  "java.lang.String",
				  "State: state string.",
				  true,
				  true,
				  false,
				  stateDesc);

	// TotalAccesses
	Descriptor totalAccessesDesc = new DescriptorSupport();
			totalAccessesDesc.setField("name","TotalAccesses");
			totalAccessesDesc.setField("descriptorType", "attribute");
			totalAccessesDesc.setField("default", "0");
			totalAccessesDesc.setField("displayName","Total Accesses on Server");
			totalAccessesDesc.setField("getMethod","getTotalAccesses");
			totalAccessesDesc.setField("setMethod","setTotalAccesses");
			Descriptor TotalAccessesMap = new DescriptorSupport(new String[] {
			    "SNMP=1.3.6.9.12.15.18.21.0",
			    "CIM=ManagedResource.Version"});
			totalAccessesDesc.setField("protocolMap",(TotalAccessesMap));

	dAttributes[1] = new ModelMBeanAttributeInfo("TotalAccesses",
			"java.lang.Integer",
			"TotalAccesses: number of times the State string",
			true,
			false,
			false,
			totalAccessesDesc);

	// MaxPool static value									
		Descriptor maxPoolHardValueDesc = new DescriptorSupport();
			maxPoolHardValueDesc.setField("name","maxPoolHardValue");
			maxPoolHardValueDesc.setField("descriptorType","attribute");
			maxPoolHardValueDesc.setField("value", new Integer("99"));
			maxPoolHardValueDesc.setField("displayName","maximum Thread Pool for Apache ServerHardCodedValue");
			maxPoolHardValueDesc.setField("currencyTimeLimit","-1");
	
		dAttributes[2] = new ModelMBeanAttributeInfo("maxPoolHardValue",
					"java.lang.Integer",
					"maxPoolHardValue: static maximum thread pool value for the Apache Serverin ModelMBeanInfo and not in TestBean",
					true,
					false,
					false,
					maxPoolHardValueDesc);


	// StartDate as a static Value
		Descriptor startDateDesc = new DescriptorSupport();
			startDateDesc.setField("name","startDate");
			startDateDesc.setField("descriptorType","attribute");
			startDateDesc.setField("displayName","ApacheServerStartTime");
			startDateDesc.setField("value", new Date());
			startDateDesc.setField("currencyTimeLimit","-1");

	// set the attributeInfo would look like this:

		dAttributes[3] = new ModelMBeanAttributeInfo("startDateHardValue",
					"java.util.Date",
					"startDate of ApacheServerHardValue: static Date value",
					true,
					false,
					false,
					startDateDesc);

	// Retrieving these attributes using the ModelMBean myMMBean:
	RequiredModelMBean myMMBean = new RequiredModelMBean();
	// set ModelMBeanInfo and managedResource here 
	
	String myState = (String) myMMBean.getAttribute("State");
	String myAccesses = (String) myMMBean.getAttribute("TotalAccesses");
	Integer myHardValue = (Integer) myMMBean.getAttribute("maxPoolHardValue");

	// Constructor Descriptor example

	ModelMBeanConstructorInfo[] dConstructors = new ModelMBeanConstructorInfo[1];
	Class apacheServerClass = null;
	try {
		apacheServerClass = Class.forName("net.jnjmx.ch4.ApacheServer");
	} catch(Exception e) {
		System.out.println("Apache Server Class not found");
	}
	
	Constructor[] constructors = apacheServerClass.getConstructors();
			
	Descriptor apacheServerDesc = new DescriptorSupport();
	apacheServerDesc.setField("name","ApacheServer");
	apacheServerDesc.setField("descriptorType", "operation");
	apacheServerDesc.setField("role","constructor");
			
	dConstructors[0] = new ModelMBeanConstructorInfo(
			"ApacheServer(): Constructs a Apache Server query tool",
			 constructors[0],
			 apacheServerDesc);
			
	// Operation Descriptor Example
	
	ModelMBeanOperationInfo[] dOperations = new ModelMBeanOperationInfo[1];
	MBeanParameterInfo[] params = null;
			
	Descriptor stopDesc = new DescriptorSupport();
	stopDesc.setField("name","stop");
	stopDesc.setField("descriptorType","operation");
	stopDesc.setField("class","apacheServer");
	stopDesc.setField("role","operation");
			
	dOperations[0] = new ModelMBeanOperationInfo("stop",
			 "stop(): stop the apache server",
			 params ,
		    "void",
			 MBeanOperationInfo.ACTION,
		     stopDesc);

	Descriptor startDesc = new DescriptorSupport(new String[] {
		    "name=start",
			"class=apacheServer",
			"descriptorType=operation",
			"role=operation",
			"currencyTimeLimit=3"} );
			
	ApacheServer asf = new ApacheServer(new String("http://www.ibm.com/hk"));
	startDesc.setField("targetObject",asf);
	startDesc.setField("targetObjectType","objectReference");
			
	dOperations[1] = new ModelMBeanOperationInfo("start",
					"start(): start the apache server",
					params ,
					"java.lang.Integer",
					MBeanOperationInfo.INFO,
					startDesc);

	// invoking the operation
	Integer stopRC = (Integer) myMMBean.invoke("stop", 
		(new Object[0]),  
		(new String[0]) );


	// Notification Descriptor Example
	ModelMBeanNotificationInfo[] dNotifications = new ModelMBeanNotificationInfo[1];
	
	Descriptor genEventDesc = new DescriptorSupport(new String[] {
		"descriptorType=notification", 
		"name=jmx.ModelMBean.General", 
		"severity=4", 
		"MessageId=MA001", 
		"log=T", 
		"logfile=jmx.log"});

	String[] genTypes = new String[1];
	genTypes[0] = new String("jmx.ModelMBean.General");

	dNotifications[0] = new ModelMBeanNotificationInfo(genTypes,
		 "jmx.ModelMBean.General", // was Generic
		 "Generic Event",
		 genEventDesc); // test event

	// Here is how you would send this Notification from the myMMBean ModelMBean:
	myMMBean.sendNotification("jmx.ModelMBean.GeneralApache Server is Stopping");


	// Custom Descriptor Example
	Descriptor totalAccessesCustomDesc = new DescriptorSupport();
	totalAccessesCustomDesc.setField("name","TotalAccesses");
	totalAccessesCustomDesc.setField("descriptorType","attribute");
	totalAccessesCustomDesc.setField("displayName","NumberChanges");
	totalAccessesCustomDesc.setField("getMethod","getTotalAccesses");
	totalAccessesCustomDesc.setField("setMethod","setTotalAccesses");
	totalAccessesCustomDesc.setField("classification","metric");  // custom descriptor

	////////////////////////////////////////////


	// Create ModelMBeanInfo		
	ModelMBeanInfo mmBeanInfo = new ModelMBeanInfoSupport("SessionPool1",
			"ModelMBeanInfo for managing todd Sessions1",
			dAttributes,
			dConstructors,
			dOperations,
			dNotifications);
			
	// set the descriptor for the ModelMBeanInfo		
	try {
	  mmBeanInfo.setMBeanDescriptor(mmbDescription);
	} catch (Exception e) {
		System.out.println("CreateModelMBeanInfo failed with " + e.getMessage());
	}		
		
	return mmBeanInfo;
	}
}
