package net.jnjmx.ch8;
/**
 * @author - Heather Kreger, Ward Harold
 * Java and JMX: Building Manageable Systems, Chapter 9
 * 
 * The program is provided "as is" without any warranty express or implied,
 * including the warranty of non-infringement and the implied warranties of
 * merchantibility and fitness for a particular purpose. The author will not be
 * liable for any damages suffered by you or any third party claim against 
 * you regarding the Program.
 */

import java.io.IOException;
import java.security.PrivilegedAction;

import javax.management.InstanceAlreadyExistsException;
import javax.management.MBeanException;
import javax.management.MBeanRegistrationException;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.MalformedObjectNameException;
import javax.management.NotCompliantMBeanException;
import javax.management.ObjectName;
import javax.management.ReflectionException;
import javax.security.auth.Subject;
import javax.security.auth.login.LoginContext;

public class spma {
    public static void main(String[] args) throws Exception {
        LoginContext lc = new LoginContext("NT");
        lc.login();

		System.out.println(lc.getSubject());
		
        Subject.doAs(lc.getSubject(), new PrivilegedAction() {
            public Object run() {
                try {
                    doSpma();
                } catch (Exception x) {
                    System.err.println(x.toString());
                }
                return null;
            }});
    }

    private static void doSpma()
        throws
            MalformedObjectNameException,
            ReflectionException,
            InstanceAlreadyExistsException,
            MBeanRegistrationException,
            MBeanException,
            NotCompliantMBeanException,
            IOException,
            Exception {
        ObjectName pmname = new ObjectName("ch8:id=ProcessManager");
        MBeanServer mbs = setupProcessManager(pmname);
        char cmd;
        System.out.print("% ");
        while ((cmd = readcmd()) != 'q') {
            switch (cmd) {
                case 'l' :
                    doListCmd(mbs, pmname);
                    break;
                case 'k' :
                    doKillCmd(mbs, pmname);
                    break;
                default :
                    usage(cmd);
            }
            System.out.print("% ");
        }
    }

    private static void doListCmd(MBeanServer mbs, ObjectName pmname)
        throws Exception {
        int[] pids = (int[]) mbs.getAttribute(pmname, "Pids");
        System.out.print("pids: [");
        for (int i = 0; i < pids.length; i++) {
            System.out.print(pids[i] + ", ");
        }
        System.out.println("]");
    }

    private static void doKillCmd(MBeanServer mbs, ObjectName pmname)
        throws Exception {
        try {
            int pid = readpid();
            mbs.invoke(
                pmname,
                "kill",
                new Object[] { new Integer(pid)},
                new String[] { "int" });
        } catch (IOException x) {
            System.out.println("Error: bad pid");
        }
    }

    private static char readcmd() throws IOException {
        char result;
        do {
            result = (char) System.in.read();
        } while ((result == '\r') || (result == '\n'));
        return result;
    }

    private static int readpid() throws IOException {
        char c = (char) System.in.read();
        if (c != ' ')
            throw new IOException();

        StringBuffer buf = new StringBuffer();
        c = (char) System.in.read();
        while (Character.isDigit(c)) {
            buf.append(c);
            c = (char) System.in.read();
        }

        return Integer.parseInt(buf.toString());
    }

    private static MBeanServer setupProcessManager(ObjectName pmname)
        throws
            ReflectionException,
            InstanceAlreadyExistsException,
            MBeanRegistrationException,
            MBeanException,
            NotCompliantMBeanException {
        MBeanServer mbs = MBeanServerFactory.createMBeanServer();
        mbs.createMBean("net.jnjmx.ch8.mbeans.ProcessManager", pmname);
        return mbs;
    }

    private static void usage(char cmd) {
        System.out.println("Unrecognized command: " + cmd);
        System.out.println("Valid commands are: l, k <pid>");
    }
}
