package net.jnjmx.ch9;
import java.io.IOException;
import java.lang.reflect.*;
import java.util.Iterator;
import javax.management.*;

/**
 */
public class CatalogManagerDD implements DynamicMBean  {
	// TackCatalogMBean Methods
	TackCatalogDD catalog;
	
	
	public CatalogManagerDD (Catalog inCatalog) { 
		catalog = (TackCatalogDD)inCatalog;
	}
	
	/* MBean Methods */
	public Object getAttribute(String attributeName) 
	throws AttributeNotFoundException,
		 MBeanException,
		 ReflectionException
	{
		if ( attributeName.equals("AutoCheckpointing") )
			return new Boolean(catalog.isAutoCheckpointing());
		if ( attributeName.equals("NumberOfItems") )
			return new Integer(catalog.getStats("NumberOfItems"));
		if ( attributeName.equals("Accesses") )
			return new Integer(catalog.getStats("Accesses"));
		if ( attributeName.equals("TotalSales") )
			return new Integer(catalog.getStats("TotalSales"));
		if ( attributeName.equals("InventorySize") )
			return new Integer(catalog.getStats("InventorySize"));
		else throw(new AttributeNotFoundException("Attribute " + attributeName + " not found in " + this.getClass().getName()));
	}

	public AttributeList getAttributes(String[] attributes) {
		// in this simple example we always return all attributes
		// in a production example, you need to only return the
		// attributes in the list passed in
		AttributeList returnList = new AttributeList();
		if ( (attributes == null) || (attributes.length == 0) || (attributes.length > 0)) {
			returnList.add(new Attribute("AutoCheckpointing",  new Boolean(catalog.isAutoCheckpointing())) );
			returnList.add(new Attribute("NumberOfItems", new Integer(catalog.getStats("NumberOfItems"))));
			returnList.add(new Attribute("Accesses", new Integer(catalog.getStats("Accesses"))) );
			returnList.add(new Attribute("TotalSales", new Integer(catalog.getStats("TotalSales"))) );
			returnList.add(new Attribute("InventorySize", new Integer(catalog.getStats("InventorySize"))) );
		}
		return returnList;

	}

	public void setAttribute(Attribute newAttribute) 
	throws AttributeNotFoundException,
		 InvalidAttributeValueException,
		 MBeanException, 
		 ReflectionException 
	{
		String attributeName = newAttribute.getName();
		if ( attributeName.equals("Autocheckpointing") )
			catalog.setAutoCheckpoint(((Boolean)(newAttribute.getValue())).booleanValue());
		else if ( (attributeName.equals("NumberOfItems")) || 
				  (attributeName.equals("TotalSales" )) ||
				  (attributeName.equals("Accesses" )) ||
				  (attributeName.equals("InventorySize")) ) {
			throw(new AttributeNotFoundException("Attribute " + attributeName + " not updateable in " + this.getClass().getName())); 
		} else
			throw(new AttributeNotFoundException("Attribute " + attributeName + " not found in " + this.getClass().getName()));

	}

	public AttributeList setAttributes(AttributeList newAttributes) {
		AttributeList returnList = new AttributeList();
		if ( newAttributes == null || newAttributes.isEmpty() )
			return returnList;

		for ( Iterator I=newAttributes.iterator(); I.hasNext(); ) {
			Attribute newAttribute= (Attribute)I.next();
			try {
				setAttribute(newAttribute);
				returnList.add(new Attribute(newAttribute.getName(), 
											newAttribute.getValue() ) );
			} catch ( Exception e ) {
			}

			return returnList;
		}
		return returnList;
	}

	public Object invoke(String operationName, Object[] parms, String[] sig) {
		try {
		if ( operationName.equals("doCheckpoint") ) {
			catalog.doCheckpoint();
		} else if ( operationName.equals("load") ) {
			try {
				return catalog.load((String)(parms[0]));
			}
			catch (Exception e) {
				System.out.println("Load of TackCatalog failed");
			}
			return null;
		} else {
			throw new 
				ReflectionException(
				new NoSuchMethodException(operationName),
				"Cannot find the operation " + operationName + 
				" in TackCatalog");
		}
		}
		catch (ReflectionException e) {
			System.out.println("Load of TackCatalog failed");
		}
		return null;
	}

	public MBeanInfo getMBeanInfo() {
	/* Attributes: boolean AutoCheckpointing r/w, 
	int numberOfItems r/o, 
	int Accesses r/o, 
	int Sales r/o 
	int InventorySize r/o */
	/* operations: void saveCatalog, 
	Catalog load, 
	void doCheckpoint */
	/* notifications: none */
		String catDescription = "Tack Catalog Dynamic MBean";
		MBeanAttributeInfo[] catAttributes = new MBeanAttributeInfo[5];
		catAttributes[0] = new MBeanAttributeInfo("AutoCheckpointing",
												  "java.lang.Boolean",
												  "AutoCheckpointing: boolean value if checkpointing of catalog is done automatically",
												  true,true,false);


		catAttributes[1] = new MBeanAttributeInfo("NumberOfItems",
												  "java.lang.Integer",
												  "NumberOfItems: number of items in the catalog",
												  true,false,false);

		catAttributes[2] = new MBeanAttributeInfo("Accesses",
												  "java.lang.Integer",
												  "Accesses: number of time items in the catalog database have been listed",
												  true,false,false);

		catAttributes[3] = new MBeanAttributeInfo("TotalSales",
												  "java.lang.Integer",
												  "Sales: number of items sold",
												  true,false,false);
												  
		catAttributes[4] = new MBeanAttributeInfo("InventorySize",
												  "java.lang.Integer",
												  "InventorySize: Total number of all items currently in inventory",
												  true,true,false);

		MBeanConstructorInfo[] catConstructors = new MBeanConstructorInfo[2];
		
		
		Constructor[] constructors = catalog.getClass().getConstructors();
		
		catConstructors[0] = new MBeanConstructorInfo(
								"tackCatalog(dbName): Constructs a tackCatalog from an existing database", 
								constructors[0]);

		MBeanOperationInfo[] catOperations = new MBeanOperationInfo[2];
 
		MBeanParameterInfo[] params = new MBeanParameterInfo[0];

		catOperations[0] = new MBeanOperationInfo("doCheckpoint",
												  "doCheckpoint(): saves the catalog to a file",
												  params , 
												  "void", 
												  MBeanOperationInfo.ACTION);
       	 params =
			new MBeanParameterInfo[] {
				(new MBeanParameterInfo("name",
				"java.lang.String",
				"Tack Catalog name"))};
				    
		catOperations[1] = new MBeanOperationInfo("load",
												  "load(dbName): loads the catalog from a file",
												  params , 
												  "void", 
												  MBeanOperationInfo.ACTION);
												  
		MBeanNotificationInfo[] catNotifications = new MBeanNotificationInfo[0];
		
		String className = this.getClass().getName();
		
		MBeanInfo catMBeanInfo = new MBeanInfo(className,
									 catDescription,
									 catAttributes,
									 catConstructors,
									 catOperations,
									 catNotifications );
									 
		return catMBeanInfo;
	} // getMBeanInfo	

}
