package net.jnjmx.ch9;


import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import java.lang.reflect.*;

import javax.management.*;
import javax.management.modelmbean.*;

// Publish Only Model MBean Example

public class TackCatalogPO implements Catalog, Serializable {

	private static final String FILEEXT = ".catalog";
	
	private final String name;

	private boolean autoCheckpoint = false;
	private Map categories;
	private List items;
	private int nextCatalogNo = 1;
	private int accesses=0, sales=0, inventorySize=0;
	ModelMBean catMBean = null;
	ObjectName dynTackMBean = null;
	MBeanServer mbs;

	public TackCatalogPO(String inName) {
		this.name = inName;
		this.categories = new HashMap();
		this.items = new ArrayList();
		try {
		dynTackMBean = new ObjectName("Chapter9:id=TackCatalogPO");
		this.catMBean = this.createCatalogMBean();
		
		ArrayList mbservers = MBeanServerFactory.findMBeanServer(null);
		if (mbservers.size() == 1) 
			mbs =  (MBeanServer)mbservers.get(0);
		else 
			System.out.println("Creation of Management Bean failed: too many MBeanServers");
		mbs.registerMBean(catMBean, dynTackMBean);
		} catch (Exception e) {
			System.out.println("Creation of Management Bean failed");
		}
	}
	
	public static TackCatalogPO create(String inName) throws IOException {
		String cfn = inName + FILEEXT; 
		File cf = new File(cfn);
		if (cf.exists()) {
			cf.delete();
		}

		TackCatalogPO catalog = new TackCatalogPO(inName);
//		Should write to a file here
//		ObjectOutputStream db =
//			new ObjectOutputStream(new FileOutputStream(cf));
//		db.writeObject(catalog);
//		db.flush();
//		db.close();

		return catalog;
	}
	
	// load is exposed in TackCatalogMBean

	public Catalog load(String name)
		throws IOException, ClassNotFoundException {
		String cfn = name + FILEEXT;
		File cf = new File(cfn);
		if (cf.exists() == false) {
			throw new FileNotFoundException(cfn + " doesn't exist");
		}

		ObjectInputStream db = new ObjectInputStream(new FileInputStream(cf));
		TackCatalogPO catalog = (TackCatalogPO) db.readObject();
		db.close();

		return catalog;
	}


	public synchronized Map getCategories() {
		return this.categories;
	}



	public synchronized void classify(Item item, Category category) {
		List items = ((TackCategory) category).getItems();
		if (items.contains(item) == false) {
			items.add(item);
		}
	}

	public synchronized Category createCategory(String name)
		throws CategoryExistsException {
		TackCategory result = new TackCategory(this, name);
		if (this.categories.get(name) != null) {
			throw new CategoryExistsException();
		}
		this.categories.put(name, result);
		return result;
	}

	public synchronized Item createItem(String name, int price, int quantity)
		throws ItemExistsException {
		TackItem result = new TackItem(this, nextCatalogNo++, name, price, quantity);
		if (this.items.contains(result)) {
			throw new ItemExistsException();
		}
		this.items.add(result);
		
		inventorySize = inventorySize + quantity;
		
		/* Update cached value in ModelMBean model MBean */
		try {
			ModelMBeanInfo localInfo = (ModelMBeanInfo)catMBean.getMBeanInfo();
			Descriptor newNum = localInfo.getDescriptor("InventorySize","attribute");
			newNum.setField("value", new Integer(inventorySize));
			localInfo.setDescriptor(newNum, "attribute");
			
			Descriptor newNumI = localInfo.getDescriptor("NumberOfItems","attribute");
			newNumI.setField("value", new Integer(items.size()));
			localInfo.setDescriptor(newNumI, "attribute");
			catMBean.setModelMBeanInfo(localInfo);
		} catch (Exception e) {
			System.out.println("Update of Management data failed");
		}
				
		return result;
	}
	
	public synchronized void declassify(Item item, Category category) {
		List items = category.getItems();
		items.remove(item);
	}

	public synchronized void decreaseInventory(Item item, int amount)
		throws InventoryUnderflowException {
		inventorySize = inventorySize - amount;
		sales = sales + amount;
		
		try {
			ModelMBeanInfo localInfo = (ModelMBeanInfo)catMBean.getMBeanInfo();
			
			Descriptor newNum = localInfo.getDescriptor("InventorySize","attribute");
			newNum.setField("value", new Integer(inventorySize));
			localInfo.setDescriptor(newNum, "attribute");
			
			Descriptor newNumS = localInfo.getDescriptor("TotalSales","attribute");
			newNumS.setField("value", new Integer(sales));
			localInfo.setDescriptor(newNumS, "attribute");
			
			catMBean.setModelMBeanInfo(localInfo);
		} catch (Exception e) {
			System.out.println("Update of Management data failed");
		}
		
		int nq = item.getQuantity() - amount;
		if (nq < 0) {
			throw new InventoryUnderflowException();
		}
		((TackItem)item).setQuantity(nq); 
	}

	public void destroy() throws IOException {
		String cfn = this.name + FILEEXT;
		File cf = new File(cfn);
		if (cf.exists()) {
			cf.delete();
		}
	}

	public synchronized Category getCategory(String name) {
		Category result = (Category) this.categories.get(name);
		return result;
	}

	public synchronized void increaseInventory(Item item, int amount) {
		inventorySize = inventorySize + amount;
		if (amount < 0) {
			throw new IllegalArgumentException();
		}
		
		try {
			ModelMBeanInfo localInfo = (ModelMBeanInfo)catMBean.getMBeanInfo();
			Descriptor newNum = localInfo.getDescriptor("InventorySize","attribute");
			newNum.setField("value", new Integer(inventorySize));
			localInfo.setDescriptor(newNum, "attribute");
			catMBean.setModelMBeanInfo(localInfo);
		} catch (Exception e) {
			System.out.println("Update of Management data failed");
		}
		
		int nq = item.getQuantity() + amount;
		((TackItem) item).setQuantity(nq);
	}



	public synchronized List getItems() {
		accesses++;
		try {
			ModelMBeanInfo localInfo = (ModelMBeanInfo)catMBean.getMBeanInfo();
			Descriptor newNum = localInfo.getDescriptor("Accesses","attribute");
			newNum.setField("value", new Integer(accesses));
			localInfo.setDescriptor(newNum, "attribute");
			
			Descriptor newNumI = localInfo.getDescriptor("NumberOfItems","attribute");
			newNumI.setField("value", new Integer(this.items.size()));
			localInfo.setDescriptor(newNumI, "attribute");
			catMBean.setModelMBeanInfo(localInfo);
		} catch (Exception e) {
			System.out.println("Update of Management data failed");
		}
		return this.items;
	}

	public synchronized void removeCategory(Category category)
		throws NoSuchCategoryException {
		removeCategory(category.getName());
	}

	public synchronized void removeCategory(String name)
		throws NoSuchCategoryException {
		Category category = (Category) this.categories.remove(name);
		if (category == null) {
			throw new NoSuchCategoryException();
		}
	}

	public synchronized void removeItem(Item item) throws NoSuchItemException {
		boolean exists = this.items.remove(item);
		if (exists == false) {
			throw new NoSuchItemException();
		}
		inventorySize = inventorySize - item.getQuantity();
		
		try {
			ModelMBeanInfo localInfo = (ModelMBeanInfo)catMBean.getMBeanInfo();
			Descriptor newNum = localInfo.getDescriptor("InventorySize","attribute");
			newNum.setField("value", new Integer(inventorySize));
			localInfo.setDescriptor(newNum, "attribute");
			
			Descriptor newNumI = localInfo.getDescriptor("NumberOfItems","attribute");
			newNumI.setField("value", new Integer(items.size()));
			localInfo.setDescriptor(newNumI, "attribute");
			catMBean.setModelMBeanInfo(localInfo);
		} catch (Exception e) {
			System.out.println("Update of Management data failed");
		}
		
		removeItemFromCategories(item);
	}

	public synchronized void removeItem(int catalogno)
		throws NoSuchItemException {
		boolean removed = false;
		Iterator i = this.items.iterator();
		while (i.hasNext()) {
			Item item = (Item) i.next();
			if (item.getCatalogNo() == catalogno) {
				this.items.remove(item);
				inventorySize = inventorySize - item.getQuantity();
				
				try {
					ModelMBeanInfo localInfo = (ModelMBeanInfo)catMBean.getMBeanInfo();
					Descriptor newNum = localInfo.getDescriptor("InventorySize","attribute");
					newNum.setField("value", new Integer(inventorySize));
					localInfo.setDescriptor(newNum, "attribute");
					
					Descriptor newNumI = localInfo.getDescriptor("NumberOfItems","attribute");
					newNumI.setField("value", new Integer(items.size()));
					localInfo.setDescriptor(newNumI, "attribute");
					catMBean.setModelMBeanInfo(localInfo);
				} catch (Exception e) {
					System.out.println("Update of Management data failed");
				}
				
				removeItemFromCategories(item);
				removed = true;
				break;
			}
		}
		if (removed == false) {
			throw new NoSuchItemException();
		}	
	}
	
	private void removeItemFromCategories(Item item) {
		Iterator i = this.categories.keySet().iterator();
		while (i.hasNext()) {
			String category = (String) i.next();
			declassify(item, (Category) this.categories.get(category));
		}
	}
	
	// TackCatalogMBean Methods

	public boolean isAutoCheckpointing() {
		try {
			ModelMBeanInfo localInfo = (ModelMBeanInfo)catMBean.getMBeanInfo();
			Descriptor newNum = localInfo.getDescriptor("AutoCheckpointing","attribute");
			newNum.setField("value", new Boolean(this.autoCheckpoint));
			localInfo.setDescriptor(newNum, "attribute");
			catMBean.setModelMBeanInfo(localInfo);
		} catch (Exception e) {
			System.out.println("Update of Management data failed");
		}
		return this.autoCheckpoint;
	}
	
	public void setAutoCheckpoint(boolean autoCheckpoint) {
		this.autoCheckpoint = autoCheckpoint;
		try {
			ModelMBeanInfo localInfo = (ModelMBeanInfo)catMBean.getMBeanInfo();
			Descriptor newNum = localInfo.getDescriptor("AutoCheckpointing","attribute");
			newNum.setField("value", new Boolean(this.autoCheckpoint));
			localInfo.setDescriptor(newNum, "attribute");
			catMBean.setModelMBeanInfo(localInfo);
		} catch (Exception e) {
			System.out.println("Update of Management data failed");
		}
	}

	public void doCheckpoint() {
		if (this.autoCheckpoint == true) {
			checkpoint();
		}
	}

	public void checkpoint() {
		// This method should save the catalog to a file
	}
	
	ModelMBean createCatalogMBean() {
	  ModelMBean catalogMBean = null;
	  try {
		catalogMBean = new RequiredModelMBean();
		catalogMBean.setModelMBeanInfo(createCatalogMBeanInfo());
		catalogMBean.setManagedResource(this, "ObjectReference");
	  }
	  catch (Exception e) {
	  	System.out.println("Creating Tack Catalog MBean failed: " + e.getMessage());
	  }
	  return catalogMBean;
	}

	ModelMBeanInfo createCatalogMBeanInfo() {
		ModelMBeanInfo info = null;
	/* Attributes: boolean AutoCheckpointing r/w, 
	int numberOfItems r/o, 
	int Accesses r/o, 
	int Sales r/o 
	int InventorySize r/o */
	/* operations: none */ 
	/* notifications: none */
		String catDescription = "Tack Catalog Dynamic MBean";
		
		Descriptor catDescriptor =
			new DescriptorSupport(
				new String[] {
					("name=TackCatalog"),
					"descriptorType=mbean",
					("displayName=TackCatalogManager"),
					"type=net.jnjmx.ch9.TackCatalogPO",
					"currencyTimeLimit=-1" });  // return cache values
		
		ModelMBeanAttributeInfo[] catAttributes = new ModelMBeanAttributeInfo[5];


		Descriptor catDescAuto =
			new DescriptorSupport(
				new String[] {
					"name=AutoCheckpointing",
					"descriptorType=attribute",
					"displayName=AutoCheckpointing",
					"currencyTimeLimit=-1" });  				// return cache values
		catDescAuto.setField("value", new Boolean(this.autoCheckpoint));  // initial value
					
		// not writable
		catAttributes[0] = new ModelMBeanAttributeInfo("AutoCheckpointing",
												  "java.lang.Boolean",
												  "AutoCheckpointing: boolean value if checkpointing of catalog is done automatically",
												  false,false,true, 
												  catDescAuto);

		Descriptor catDescItems =
			new DescriptorSupport(
				new String[] {
					"name=NumberOfItems",
					"descriptorType=attribute",
					"displayName=NumberOfItems",
					"currencyTimeLimit=-1" });  		  // return cache values
		catDescItems.setField("value", new Integer(0));  // initial value

		catAttributes[1] = new ModelMBeanAttributeInfo("NumberOfItems",
												  "java.lang.Integer",
												  "NumberOfItems: number of items in the catalog",
												  true,false,false,
												  catDescItems);
		Descriptor catDescAccess =
			new DescriptorSupport(
				new String[] {
					"name=Accesses",
					"descriptorType=attribute",
					"displayName=Accesses",
					"currencyTimeLimit=-1" });  		  // return cache values
		catDescAccess.setField("value", new Integer(0));  // initial value

		catAttributes[2] = new ModelMBeanAttributeInfo("Accesses",
												  "java.lang.Integer",
												  "Accesses: number of time items in the catalog database have been listed",
												  true,false,false,
												  catDescAccess);

		Descriptor catDescSales =
			new DescriptorSupport(
				new String[] {
					"name=TotalSales",
					"descriptorType=attribute",
					"displayName=TotalSales",
					"currencyTimeLimit=-1" });  		  // return cache values
		catDescSales.setField("value", new Integer(0));  // initial value

		catAttributes[3] = new ModelMBeanAttributeInfo("TotalSales",
												  "java.lang.Integer",
												  "Sales: number of items sold",
												  true,false,false,
												  catDescSales);
												  
		Descriptor catDescInv =
			new DescriptorSupport(
				new String[] {
					"name=InventorySize",
					"descriptorType=attribute",
					"displayName=InventorySize",
					"currencyTimeLimit=-1" });         // always cache
		catDescInv.setField("value", new Integer(0));  // initial value
												  
		catAttributes[4] = new ModelMBeanAttributeInfo("InventorySize",
												  "java.lang.Integer",
												  "InventorySize: Total number of all items currently in inventory",
												  true,false,false,
												  catDescInv);

		ModelMBeanConstructorInfo[] catConstructors = new ModelMBeanConstructorInfo[1];
		
		Constructor[] constructors = this.getClass().getConstructors();
		
		Descriptor tackConstDesc =
			new DescriptorSupport(
				new String[] {
					"name=TackCatalogPO",
					"descriptorType=operation",
					"role=constructor" });
					
		catConstructors[0] = new ModelMBeanConstructorInfo(
								"tackCatalog(dbName): Constructs a tackCatalog from an existing database", 
								constructors[0],
								tackConstDesc);

		ModelMBeanOperationInfo[] catOperations = new ModelMBeanOperationInfo[0];
 
					  
		ModelMBeanNotificationInfo[] catNotifications = new ModelMBeanNotificationInfo[0];
		
		String className = this.getClass().getName();
		
		info = new ModelMBeanInfoSupport(className,
									 catDescription,
									 catAttributes,
									 catConstructors,
									 catOperations,
									 catNotifications );
		try {
			info.setMBeanDescriptor(catDescriptor);
		} catch (Exception e) {
			System.out.println("CreateModelMBeanInfo failed with " + e.getMessage());
		}

		return info;
	}

}	// class

