package net.jnjmx.ch9;

import java.util.HashMap;
import java.util.Set;

import javax.management.MBeanInfo;
import javax.management.MBeanAttributeInfo;
import javax.management.MBeanConstructorInfo;
import javax.management.MBeanNotificationInfo;
import javax.management.MBeanOperationInfo;
import javax.management.MBeanParameterInfo;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.ObjectName;
import javax.management.modelmbean.ModelMBean;

public class TackStore {
    private static HashMap types;

    static {
	types = new HashMap();
	types.put("standard", "net.jnjmx.ch9.TackCatalog");
	types.put("dynamic", "net.jnjmx.ch9.TackCatalogD");
	types.put("model", "net.jnjmx.ch9.TackCatalogMM");
    };

    public static void main(String[] args) throws Exception {
	MBeanServer mbs = MBeanServerFactory.createMBeanServer();
	String mbtype = getMBeanType();
	ObjectName cmbname = createMBeanForType(mbs, mbtype);
	MBeanInfo mbinfo = mbs.getMBeanInfo(cmbname);
	dumpMBeanInfo(mbs, cmbname, mbinfo);
    }

    private static ObjectName createMBeanForType(MBeanServer mbs,
						 String mbtype)
	throws Exception {
	ObjectName oname = new ObjectName("catalog:type=" + mbtype);
	if (mbtype.compareToIgnoreCase("standard") == 0 ||
	    mbtype.compareToIgnoreCase("dynamic") == 0) {
	    String mbclass = (String) TackStore.types.get(mbtype);
	    mbs.createMBean(mbclass,
			    oname,
			    new Object[] { "TackStoreCatalog" },
			    new String[] { "java.lang.String" });
	} else {
	    TackCatalogMM tcmm = new TackCatalogMM("TackStoreCatalog");
	    ModelMBean mmbean = tcmm.createCatalogMBean();
	    mbs.registerMBean(mmbean, oname);
	}
	return oname;
    }

    private static void dumpMBeanInfo(MBeanServer mbs, ObjectName oname, MBeanInfo mbi) {
	System.out.println("Object Name\n\t" + oname.toString());
	System.out.print("Constructors\n" + dumpConstructors(mbs, oname, mbi));
	System.out.print("Operations\n" + dumpOperations(mbs, oname, mbi));
	System.out.print("Attriubtes\n" + dumpAttributes(mbs, oname, mbi));
	System.out.print("Notifications\n" + dumpNotifications(mbs, oname, mbi));
    }

    private static String dumpAttributes(MBeanServer mbs, ObjectName oname, MBeanInfo mbi) {
	MBeanAttributeInfo[] attrs = mbi.getAttributes();
	StringBuffer sb = new StringBuffer();
	if (attrs == null || attrs.length == 0) {
	    sb.append("\tnone\n");
	} else {
	    for (int i = 0; i < attrs.length; i++) {
		sb.append("\t");
			
		if (attrs[i].isIs()) sb.append("i");
		else sb.append("-");
		if (attrs[i].isReadable()) sb.append("r");
		else sb.append("-");
		if (attrs[i].isWritable()) sb.append("w");
		else sb.append("-");
			
		sb.append("\t" + attrs[i].getType() +
			  "\t" + attrs[i].getName());

		try {
		    Object attr = mbs.getAttribute(oname, attrs[i].getName());
		    sb.append("\t" + attr.toString());
		} catch (Exception x) {
		    sb.append("\tunavailable");
		}
		sb.append("\n");
	    }
	}
	return sb.toString();
    }

    private static String dumpConstructors(MBeanServer mbs, ObjectName oname, MBeanInfo mbi) {
	MBeanConstructorInfo[] ctors = mbi.getConstructors();
	StringBuffer sb = new StringBuffer();
	if (ctors == null || ctors.length == 0) {
	    sb.append("\tnone\n");
	} else {
	    for (int i = 0; i < ctors.length; i++) {
		if (ctors[i] == null) continue;
		sb.append("\t");
		String ctorname = ctors[i].getName();
		ctorname = (ctorname == null || ctorname.length() == 0) ? "unknown" : ctorname;
		sb.append(ctorname + "(");
		MBeanParameterInfo[] params = ctors[i].getSignature();
		if (params != null && params.length > 0) {
		    for (int j = 0; j < params.length; j++) {
			String type = params[j].getType();
			type = (type == null || type.length() == 0) ? "unknown" : type;
			String name = params[j].getName();
			name = (name == null || name.length() == 0) ? "param" + j : name;
			sb.append(type + " " + name);
			if (params.length > (j+1)) sb.append(", ");
		    }
		}
		sb.append(")\n");
	    }
	}
	return sb.toString();
    }

    private static String dumpNotifications(MBeanServer mbs, ObjectName oname, MBeanInfo mbi) {
	MBeanNotificationInfo[] notifs = mbi.getNotifications();
	StringBuffer sb = new StringBuffer();
	if (notifs == null || notifs.length == 0) {
	    sb.append("\tnone\n");
	} else {
	    for (int i = 0; i < notifs.length; i++) {
		sb.append("\t");
		sb.append(notifs[i].getName() + "\n");
		String[] ntypes = notifs[i].getNotifTypes();
		for (int j = 0; (ntypes != null) && (j < ntypes.length); j++) {
		    sb.append("\t\t" + ntypes[j] + "\n");
		}
	    }
	}
	return sb.toString();
    }

    private static String dumpOperations(MBeanServer mbs, ObjectName oname, MBeanInfo mbi) {
	MBeanOperationInfo[] ops = mbi.getOperations();
	StringBuffer sb = new StringBuffer();
	if (ops == null || ops.length == 0) {
	    sb.append("\tnone\n");
	} else {
	    for (int i = 0; i < ops.length; i++) {
		sb.append("\t");
		sb.append(ops[i].getReturnType() + " " +
			  ops[i].getName() + "(");
		MBeanParameterInfo[] params = ops[i].getSignature();
		if (params != null && params.length > 0) {
		    for (int j = 0; j < params.length; j++) {
			sb.append(params[j].getType() + " " +
				  params[j].getName());
			if (params.length > (j+1)) sb.append(", ");
		    }
		}
		sb.append(")\n");
	    }
	}
	return sb.toString();
    }

    private static String getMBeanType() {
	String mbtype = System.getProperty("mbean.type", "standard");
	if (TackStore.types.keySet().contains(mbtype) == false) {
	    throw new IllegalArgumentException("Unknown mbean.type " + mbtype);
	}
	return mbtype;
    }
}
