#!/bin/ksh -p
#
# File: jmxd.sh
# Sets environment variables and launches the jmxd (TMX4J daemon) 
# executable.
# Ensure all the parameters are set correctly before launching the script.
#

#
# The directory in which you want the daemon to execute.  
# This should preferably be on an unmountable file system
# Eg. export EXECUTE_DIR=/var
#
export EXECUTE_DIR=


#
# Set daemon logging output. Set to ON to use syslog. 
# Set to OFF to log to stderr.
# export LOGGING=OFF
#
export LOGGING=OFF



# The rest of the variables in this script should not need setting

# Call the script to set your TMX4J_HOME installation directory and JAVA_HOME
. ./set_tmx4j_home.sh


#
# Leave the following three variables unchanged unless you have modified 
# the original install structure
#
EXT_LIB_DIR=${TMX4J_HOME}/ext/lib
JMX_LIB_DIR=${TMX4J_HOME}/base/lib
JMX_BIN_DIR=${TMX4J_HOME}/ext/bin/solaris
SAMPLES_DIR=${TMX4J_HOME}/ext


#
# The LD_LIBRARY_PATH must include the location of the libjmxerror.so 
# and the libjvm.so files - you should be able to leave it as in the example
# Eg. export LD_LIBRARY_PATH=${LD_LIBRARY_PATH}:${JAVA_HOME}/jre/lib/sparc/:
#      ${EXT_LIB_DIR}
#
export LD_LIBRARY_PATH=${LD_LIBRARY_PATH}:${JAVA_HOME}/jre/lib/sparc/:${EXT_LIB_DIR}/solaris


#
# The CLASSPATH must include the base and extensions jar-files, the path to the
# .properties files, the JLog package, and the JDK rt.jar file.
# 
# Eg. export CLASSPATH=${JMX_LIB_DIR}/jmxc.jar:${JMX_LIB_DIR}/jmxx.jar:
#        ${EXT_LIB_DIR}/jmxext.jar:${EXT_LIB_DIR}:${JMX_LIB_DIR}:${SAMPLES_DIR}:
#        ${JAVA_HOME}/jre/lib/rt.jar:${CLASSPATH}
#
export CLASSPATH=${JMX_LIB_DIR}/jmxc.jar:${JMX_LIB_DIR}/jmxx.jar:${EXT_LIB_DIR}/jmxext.jar:${EXT_LIB_DIR}:${JMX_LIB_DIR}:${SAMPLES_DIR}:${JAVA_HOME}/jre/lib/rt.jar:${CLASSPATH}



# Now we launch the daemon
exec ${JMX_BIN_DIR}/jmxd
