/*---------------------------------------------------------------------------*\
|                                                                             |
| COPYRIGHT:                                                                  |
|   Tivoli Management Extensions for Java V1.1                                |
|                                                                             |
|   Licensed Materials - Property of IBM                                      |
|   (C) Copyright IBM Corporation 2000                                        |
|   All Rights Reserved                                                       |
|   US Government Users Restricted Rights - Use, duplication, or              |
|   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.         |
|                                                                             |
\*---------------------------------------------------------------------------*/

// $Revision: %Z% %M% %I% %W% %E% %U% $

package samples;

import samples.mbeans.MBean_Monitor;
import javax.management.*;
import javax.management.monitor.*;

/**
 *  GaugeMonitor_Agent.java
 *
 * @author Marco De Gregorio
 * @version %I%
 *
 */
public class GaugeMonitor_Agent
{
    MBeanServer mBS;
    GaugeMonitor monitor;
    ObjectName on_mbean, on_monitor;
    ObjectInstance oi_mbean, oi_monitor;
    MBean_Monitor tm;

    public GaugeMonitor_Agent()
    {   
        try {
            // Instantiates the MBean server
        
            mBS = MBeanServerFactory.createMBeanServer();
            System.out.println(">> Istantiated: MBeanServer");
        
            // Instantiates and registers an MBean in the MBean server
            
            tm = new MBean_Monitor();
            on_mbean = new ObjectName("Domain:description=Monitor_wrapper");
            oi_mbean = mBS.registerMBean(tm, on_mbean);
            System.out.println(">> Registered: MBean_Monitor");
            
            // Instantiates and registers a GaugeMonitor in the MBean server
            
            monitor = new GaugeMonitor();
            on_monitor = new ObjectName("MdGDomain:description=myMonitor");
            
            monitor.setObservedObject(on_mbean);
            monitor.setObservedAttribute("ThirdAttribute");
            monitor.setGranularityPeriod(300);
            monitor.setThresholds(new Integer(10), new Integer(5));
            monitor.setNotifyHigh(true);
            monitor.setNotifyLow(true);
            System.out.println(">> Istantiated: Monitor");
            
            // The tm MBean is added as listener to the Monitor
            
            monitor.addNotificationListener(tm, null, new String("myHandback"));
            
            oi_monitor = mBS.registerMBean(monitor, on_monitor);
            System.out.println(">> Registered: Monitor");
        }
        catch (Exception e) {
            e.printStackTrace();
            
            System.exit(1);
        }
    }
    
    public void run() {
    
      try {
        // Starts the monitor
        monitor.start();
        
        // Waits 20 sec.
        Thread.sleep(20000);
        
        // ...and then stops it
        monitor.stop();
      }
      catch (Exception e) {
        e.printStackTrace();
        System.exit(1);
      }
    }

    public static void main(String args[])
    {
        GaugeMonitor_Agent agent = new GaugeMonitor_Agent();
        agent.run();
        
        System.out.println("***  end GaugeMonitorExample  ***");
        System.exit(0);
    }
}
