/*---------------------------------------------------------------------------*\
|                                                                             |
| COPYRIGHT:                                                                  |
|   Tivoli Management Extensions for Java V1.1                                |
|                                                                             |
|   Licensed Materials - Property of IBM                                      |
|   (C) Copyright IBM Corporation 2000                                        |
|   All Rights Reserved                                                       |
|   US Government Users Restricted Rights - Use, duplication, or              |
|   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.         |
|                                                                             |
\*---------------------------------------------------------------------------*/

// $Revision: %Z% %M% %I% %W% %E% %U% $

package samples;
import com.tivoli.jmx.jar.*;
import javax.management.*;
import java.io.*;
import java.util.*;

/*******************************************************************************
 *
 * JarInstaller_Sample.java
 *
 * @author Maurizio Simeoni
 * @version %I%
 *
 ******************************************************************************/
public class JarInstaller_Sample  {
  
  /**
   *  The MBean's class name.
   *  It is packaged in the MLet_Archive.jar
   */
  private static final String MBEAN_CLASS = "samples.mbeans.MBean_MLet";

  /**
   * The ObjectName domain for the MBean 
   */
  private static final String MBEAN_DOMAIN = "mbean_domain";
  
  /**
   * The properties file where is stored the name of the directory that
   * will be monitored
   */
  private static final String MLET_PROPERTIES = "mlet";
  
  /**
   * The key in the properies.
   */
  private static final String DIRECTORY_KEY = "ServerPath1";

  /**
   * The JarInstaller MBean's ObjectName domain
   */
  private static final String JAR_DOMAIN = "jar_domain";
  
  /**
   * Loads the MBean from the MLet_Archive.jar.
   * 
   */
  public void load () {
    
    try {
      // get the directory where the jar is placed from the mlet.properties
      String jarDirectory =
        ResourceBundle.getBundle(MLET_PROPERTIES).getString(DIRECTORY_KEY);
      System.out.println("jarDire=" + jarDirectory);
      
      // create the MBean Server
      System.out.println("Create the MBean Server...");
      MBeanServer server = MBeanServerFactory.createMBeanServer();
      
      // create the JarInstaller ObjectName
      ObjectName installerObjName = 
        new ObjectName(JAR_DOMAIN ,"description", "jar");
        
     // create a JarInstaller MBean
     System.out.println("Create and register the JarInstaller MBean");
     server.createMBean(
       JarInstaller.class.getName(), installerObjName,
       new Object[] {new File(jarDirectory), JAR_DOMAIN},
       new String[] {File.class.getName(), String.class.getName()});
        
     // set the purge flag
      Attribute purge = new Attribute("Purge", Boolean.TRUE);
      server.setAttribute(installerObjName, purge);
        
      //enable the installer
      System.out.println("Enable the JarInstaller MBean");
      server.invoke(installerObjName, "enable", null, null);
      
     try {
       Thread.sleep(2000);
     }
     catch(Exception e) {
       e.printStackTrace();
     }
      // the jar should have been discoverd
      // try to create the MBean contained into the jar
      System.out.println
       ("Try to to create the " + MBEAN_CLASS + " MBean contained into the jar");
      server.createMBean
        (MBEAN_CLASS, new ObjectName(MBEAN_DOMAIN, "description", MBEAN_CLASS), 
                           new Object[] {new Byte(Byte.MIN_VALUE), "A sample"}, 
                  new String[] {byte.class.getName(), String.class.getName()});

      System.out.println("MBean created succesfully");
      
      // query the MBean Server in order to show that the MBean
      // is effectively registered in the server
      Set mbeansInstances = 
      server.queryMBeans(new ObjectName(MBEAN_DOMAIN + ":*"), null);
      Iterator iterator = mbeansInstances.iterator();
      while (iterator.hasNext()) {
        ObjectInstance instance = (ObjectInstance) iterator.next();
        System.out.println("ObjectName: [" + instance.getObjectName()+ "]");
      }
      
      //disable the installer
      System.out.println("Disable the JarInstaller MBean");
      server.invoke(installerObjName, "disable", null, null);
    }
    catch(Exception e) {
      System.out.println(e.toString());
      e.printStackTrace();
    }

  } // end load
  
  /**
   * The main
   */
  public static void main (String[] args) {
    JarInstaller_Sample installer = new JarInstaller_Sample();
    installer.load(); 
  } // end main
  
  
} // end JarInstaller_Sample class
